# -*- coding: utf-8 -*-

from decimal import *
from koalixcrm.crm.documents.sales_document_position import SalesDocumentPosition


class Calculations:

    @staticmethod
    def calculate_document_price(document, pricing_date):
        """Performs a price recalculation on contact documents.
        The calculated price is stored in the last_calculated_price and last_calculated_tax.
        The date when the price was calculated is stored in last_pricing_date

        Args:
            document: possible document classes must be derived from Contract
            pricing_date: must be a python date object

        Returns:
            1 (Boolean) when passed
            or raises exception

        Raises:
            Can trow Product.NoPriceFound when Product Price was overwritten but the price was not set
            Can trow Position.NoPriceFound when Position Price has no value but overwrite price is set """

        price = 0
        tax = 0
        positions = SalesDocumentPosition.objects.filter(sales_document=document.id)
        contact_for_price_calculation = document.customer
        if positions.exists():
            for position in positions:
                price += Calculations.calculate_position_price(position,
                                                               pricing_date,
                                                               contact_for_price_calculation,
                                                               document.currency)
                tax += Calculations.calculate_position_tax(position, document.currency)

            if document.discount is not None:
                discount = Decimal(document.discount)
                total_price = price * (1 - discount / 100)
                total_tax = tax * (1 - discount / 100)
                total_price = Decimal(total_price)
                total_tax = Decimal(total_tax)
                price = document.currency.round(total_price)
                tax = document.currency.round(total_tax)
        document.last_calculated_price = price
        document.last_calculated_tax = tax
        document.last_pricing_date = pricing_date
        document.save()
        return 1

    @staticmethod
    def calculate_position_price(position, pricing_date, contact, currency):
        """Performs a price calculation a position.
        The calculated price is stored in the last_calculated_price
        The date when the price was calculated is stored in last_pricing_date

        Args:
            position: possible document classes must be derived from Contract class
            pricing_date: must be a python date object
            contact: is used for the lookup of the correct price
            currency: is used for the lookup of the correct price and for the definition of the rounding

        Returns:
            calculated price
            or raises exception

        Raises:
            Can trow Product.NoPriceFound when Product Price was overwritten but the price was not set
            Can trow Position.NoPriceFound when Position Price has no value but overwrite price is set """


        if not position.overwrite_product_price:
            position.position_price_per_unit = position.product_type.get_price(pricing_date,
                                                                               position.unit,
                                                                               contact,
                                                                               currency)
        elif position.position_price_per_unit is None:
            raise SalesDocumentPosition.NoPriceFound
        nominal_total = position.position_price_per_unit * position.quantity
        if isinstance(position.discount, Decimal):
            nominal_minus_discount = nominal_total * (1 - position.discount / 100)
        else:
            nominal_minus_discount = nominal_total
        total_with_tax = Decimal(nominal_minus_discount)
        position.last_calculated_price = currency.round(total_with_tax)
        position.last_pricing_date = pricing_date
        position.save()
        return position.last_calculated_price

    @staticmethod
    def calculate_position_tax(position, currency):
        """Performs a tax calculation a position.
        The calculated tax is stored in the last_calculated_tax

        Args:
            position: possible document classes must be derived from Position class
            currency: is used for the definition of the rounding

        Returns:
            calculated tax
            or raises exception

        Raises:
            Can trow Product.NoPriceFound when Product Price could not be found"""
        nominal_total = position.position_price_per_unit * position.quantity
        if isinstance(position.discount, Decimal):
            nominal_minus_discount = nominal_total * (1 - position.discount / 100)
        else:
            nominal_minus_discount = nominal_total
        total_tax = nominal_minus_discount * position.product_type.get_tax_rate() / 100
        total_tax = Decimal(total_tax)
        position.last_calculated_tax = currency.round(total_tax)
        position.save()
        return position.last_calculated_tax



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-09 19:49
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import filebrowser.fields


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='subscription',
            options={'verbose_name': 'Abonnement', 'verbose_name_plural': 'Abonnemente'},
        ),
        migrations.AlterModelOptions(
            name='subscriptionevent',
            options={'verbose_name': 'Abonnement Ereignis', 'verbose_name_plural': 'Abonnement Ereignisse'},
        ),
        migrations.AlterModelOptions(
            name='subscriptiontype',
            options={'verbose_name': 'Abonnement Typ', 'verbose_name_plural': 'Abonnement Typen'},
        ),
        migrations.AlterField(
            model_name='subscription',
            name='contract',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Contract', verbose_name='Abonnement Typ'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='subscriptiontype',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='subscriptions.SubscriptionType', verbose_name='Abonnement Typ'),
        ),
        migrations.AlterField(
            model_name='subscriptionevent',
            name='event',
            field=models.CharField(choices=[('O', 'Offeriert'), ('C', 'Aufgelöst'), ('S', 'Unterschrieben')], max_length=1, verbose_name='Ereignis'),
        ),
        migrations.AlterField(
            model_name='subscriptionevent',
            name='eventdate',
            field=models.DateField(blank=True, null=True, verbose_name='Ereignis Datum'),
        ),
        migrations.AlterField(
            model_name='subscriptionevent',
            name='subscriptions',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subscriptions.Subscription', verbose_name='Abonnement'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='automaticContractExtension',
            field=models.IntegerField(blank=True, null=True, verbose_name='Automatische Vertragsverlängerung in Monaten'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='automaticContractExtensionReminder',
            field=models.IntegerField(blank=True, null=True, verbose_name='Automatische Vertragsverlängerung Erinnerung in Tagen'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='cancelationPeriod',
            field=models.IntegerField(blank=True, null=True, verbose_name='Kündigungsfirst in Monaten'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='contractDocument',
            field=filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True, verbose_name='Vertragsdokumente'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='minimumDuration',
            field=models.IntegerField(blank=True, null=True, verbose_name='Minimale Vertragsdauer'),
        ),
        migrations.AlterField(
            model_name='subscriptiontype',
            name='paymentIntervall',
            field=models.IntegerField(blank=True, null=True, verbose_name='Zahlungsintervall'),
        ),
    ]



# -*- coding: utf-8 -*-

from koalixcrm.crm.models import Quote
from koalixcrm.crm.factories.factory_sales_document import StandardSalesDocumentFactory


class StandardQuoteFactory(StandardSalesDocumentFactory):
    class Meta:
        model = Quote

    valid_until = "2018-05-20"
    status = "C"



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.core.validators import MinValueValidator
from django.utils.translation import ugettext as _


class Position(models.Model):
    position_number = models.PositiveIntegerField(verbose_name=_("Position Number"),
                                                  validators=[MinValueValidator(1)])
    quantity = models.DecimalField(verbose_name=_("Quantity"),
                                   decimal_places=3,
                                   max_digits=10)
    description = models.TextField(verbose_name=_("Description"),
                                   blank=True, null=True)
    discount = models.DecimalField(max_digits=5, decimal_places=2,
                                   verbose_name=_("Discount"),
                                   blank=True,
                                   null=True)
    product_type = models.ForeignKey("ProductType",
                                     on_delete=models.CASCADE,
                                     verbose_name=_("Product"),
                                     blank=False,
                                     null=True)
    unit = models.ForeignKey("Unit",
                             on_delete=models.CASCADE,
                             verbose_name=_("Unit"),
                             blank=True,
                             null=True)
    sent_on = models.DateField(verbose_name=_("Shipment on"),
                               blank=True,
                               null=True)
    overwrite_product_price = models.BooleanField(verbose_name=_('Overwrite Product Price'))
    position_price_per_unit = models.DecimalField(verbose_name=_("Price Per Unit"),
                                                  max_digits=17,
                                                  decimal_places=2,
                                                  blank=True, null=True)
    last_pricing_date = models.DateField(verbose_name=_("Last Pricing Date"),
                                         blank=True,
                                         null=True)
    last_calculated_price = models.DecimalField(max_digits=17,
                                                decimal_places=2,
                                                verbose_name=_("Last Calculated Price"),
                                                blank=True, null=True)
    last_calculated_tax = models.DecimalField(max_digits=17,
                                              decimal_places=2,
                                              verbose_name=_("Last Calculated Tax"),
                                              blank=True,
                                              null=True)

    def __str__(self):
        return _("Position") + ": " + self.id.__str__()

    class Meta:
        app_label = "crm"
        ordering = ["position_number"]
        verbose_name = _('Position')
        verbose_name_plural = _('Positions')


class SalesDocumentPosition(Position):
    sales_document = models.ForeignKey("SalesDocument", on_delete=models.CASCADE, verbose_name=_("Contract"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Position in Sales Document')
        verbose_name_plural = _('Positions Sales Document')

    @staticmethod
    def add_positions(position_class, object_to_create_pdf):
        from koalixcrm.crm.product.unit import Unit
        from koalixcrm.crm.product.product_type import ProductType
        objects = list(position_class.objects.filter(sales_document=object_to_create_pdf.id))
        for position in list(position_class.objects.filter(sales_document=object_to_create_pdf.id)):
            objects += list(Position.objects.filter(id=position.id))
            objects += list(ProductType.objects.filter(id=position.product_type.id))
            objects += list(Unit.objects.filter(id=position.unit.id))
        return objects

    def create_position(self, calling_model, attach_to_model):
        """Copies all the content of the calling model and attaches
        links itself to the attach_to_model, this function is usually
        used within the create invoice, quote, reminder,... functions"""

        self.product_type = calling_model.product_type
        self.position_number = calling_model.position_number
        self.quantity = calling_model.quantity
        self.description = calling_model.description
        self.discount = calling_model.discount
        self.unit = calling_model.unit
        self.sent_on = calling_model.sent_on
        self.overwrite_product_price = calling_model.overwrite_product_price
        self.position_price_per_unit = calling_model.position_price_per_unit
        self.last_pricing_date = calling_model.last_pricing_date
        self.last_calculated_price = calling_model.last_calculated_price
        self.last_calculated_tax = calling_model.last_calculated_tax
        self.sales_document = attach_to_model
        self.save()

    def __str__(self):
        return _("Sales Document Position") + ": " + str(self.id)

    class NoPriceFound(Exception):
        def __str__(self):
            return _("There is no Price set for the sales document position")


class SalesDocumentInlinePosition(admin.TabularInline):
    model = SalesDocumentPosition
    extra = 1
    classes = ['expand']
    fieldsets = (
        ('', {
            'fields': (
                'position_number',
                'quantity',
                'unit',
                'product_type',
                'description',
                'discount',
                'overwrite_product_price',
                'position_price_per_unit',
                'sent_on')
        }),
    )
    allow_add = True




# -*- coding: utf-8 -*

from django.utils.translation import ugettext as _

ACCOUNTTYPECHOICES = (
    ('E', _('Earnings')),
    ('S', _('Spendings')),
    ('L', _('Liabilities')),
    ('A', _('Assets')),
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-08-05 20:47
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0044_reportingperiod_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='work',
            name='worked_hours',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Stop Time'),
        ),
        migrations.AlterField(
            model_name='task',
            name='planned_end_date',
            field=models.DateField(blank=True, null=True, verbose_name='Planned End'),
        ),
        migrations.AlterField(
            model_name='task',
            name='planned_start_date',
            field=models.DateField(blank=True, null=True, verbose_name='Planned Start'),
        ),
        migrations.AlterField(
            model_name='task',
            name='status',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.TaskStatus', verbose_name='Status'),
        ),
        migrations.AlterField(
            model_name='work',
            name='start_time',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Start Time'),
        ),
        migrations.AlterField(
            model_name='work',
            name='stop_time',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Stop Time'),
        ),
    ]



# -*- coding: utf-8 -*-

import factory
import datetime
from koalixcrm.crm.models import Work
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.crm.factories.factory_human_resource import StandardUserExtensionFactory
from koalixcrm.global_support_functions import make_date_utc


class StandardWorkFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Work

    human_resource = factory.SubFactory(StandardUserExtensionFactory)
    date = make_date_utc(datetime.datetime(2018, 5, 1, 0, 00))
    start_time = None
    stop_time = None
    worked_hours = "1.50"
    short_description = "The employee did some work"
    description = "And here he describes some more about his work"
    task = factory.SubFactory(StandardTaskFactory)
    reporting_period = factory.SubFactory(StandardReportingPeriodFactory)



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import HumanResource
from koalixcrm.djangoUserExtension.factories.factory_user_extension import StandardUserExtensionFactory
from koalixcrm.crm.factories.factory_resource_type import StandardResourceTypeFactory
from koalixcrm.crm.factories.factory_resource_manager import StandardResourceManagerFactory


class StandardHumanResourceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = HumanResource

    user = factory.SubFactory(StandardUserExtensionFactory)
    resource_type = factory.SubFactory(StandardResourceTypeFactory)
    resource_manager = factory.SubFactory(StandardResourceManagerFactory)



# -*- coding: utf-8 -*-
import pytest
import time
import datetime
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from selenium import webdriver
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from koalixcrm.test_support_functions import *
from koalixcrm.crm.factories.factory_user import AdminUserFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.djangoUserExtension.factories.factory_user_extension import StandardUserExtensionFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.crm.reporting.work import Work
from koalixcrm.crm.factories.factory_human_resource import StandardHumanResourceFactory


class TimeTrackingWorkEntry(StaticLiveServerTestCase):
    @classmethod
    def setUpClass(cls):
        super(TimeTrackingWorkEntry, cls).setUpClass()
        firefox_options = webdriver.firefox.options.Options()
        firefox_options.set_headless(headless=True)
        cls.selenium = webdriver.Firefox(firefox_options=firefox_options)
        cls.selenium.implicitly_wait(10)
        cls.test_user = AdminUserFactory.create()
        cls.test_customer_group = StandardCustomerGroupFactory.create()
        cls.test_customer = StandardCustomerFactory.create(is_member_of=(cls.test_customer_group,))
        cls.test_user_extension = StandardUserExtensionFactory.create(user=cls.test_user)
        cls.test_human_resource = StandardHumanResourceFactory.create(user=cls.test_user_extension)
        cls.test_currency = StandardCurrencyFactory.create()
        cls.test_reporting_period = StandardReportingPeriodFactory.create()
        cls.test_1st_task = StandardTaskFactory.create(title="1st Test Task",
                                                       project=cls.test_reporting_period.project,
                                                       )

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(TimeTrackingWorkEntry, cls).tearDownClass()

    @pytest.mark.front_end_tests
    def test_delete_of_new_blank_row(self):
        selenium = self.selenium
        # login
        selenium.get('%s%s' % (self.live_server_url, '/koalixcrm/crm/reporting/time_tracking/'))
        # the browser will be redirected to the login page
        timeout = 5
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, 'id_username'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")
        username = selenium.find_element_by_xpath('//*[@id="id_username"]')
        password = selenium.find_element_by_xpath('//*[@id="id_password"]')
        submit_button = selenium.find_element_by_xpath('/html/body/div/article/div/div/form/div/ul/li/input')
        username.send_keys("admin")
        password.send_keys("admin")
        submit_button.send_keys(Keys.RETURN)
        # after the login, the browser is redirected to the target url /koalixcrm/crm/reporting/time_tracking
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, 'id_form-0-project'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")
        # find the form element
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-project"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-task"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-datetime_start_0]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-datetime_stop_0"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-datetime_start_1]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-datetime_stop_1"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-worked_hours"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-description"]')
        assert_when_element_does_not_exist(self, 'save')
        # check existence of a second form before pressing add more
        assert_when_element_exists(self, '//*[@id="id_form-1-project"]')
        add_more_button = selenium.find_element_by_xpath('//*[@id="add_more"]')
        add_more_button.send_keys(Keys.RETURN)
        time.sleep(1)
        # check existence of a second form after pressing add more
        assert_when_element_does_not_exist(self, '//*[@id="id_form-1-project"]')
        add_more_button = selenium.find_element_by_xpath('//*[@id="add_more"]')
        add_more_button.send_keys(Keys.RETURN)
        delete_form = selenium.find_element_by_id('id_form-1-DELETE')
        if not delete_form.is_selected():
            delete_form.send_keys(Keys.SPACE)
        # check existence of a second form after pressing add more
        assert_when_element_does_not_exist(self, '//*[@id="id_form-2-project"]')
        project = selenium.find_element_by_xpath('//*[@id="id_form-2-project"]')
        datetime_start_date = selenium.find_element_by_xpath('//*[@id="id_form-2-datetime_start_0"]')
        datetime_start_time = selenium.find_element_by_xpath('//*[@id="id_form-2-datetime_start_1"]')
        datetime_stop_date = selenium.find_element_by_xpath('//*[@id="id_form-2-datetime_stop_0"]')
        datetime_stop_time = selenium.find_element_by_xpath('//*[@id="id_form-2-datetime_stop_1"]')
        description = selenium.find_element_by_xpath('//*[@id="id_form-2-description"]')
        project.send_keys(self.test_reporting_period.project.id.__str__())
        datetime_start_date.send_keys(datetime.date.today().__str__())
        datetime_stop_date.send_keys(datetime.date.today().__str__())
        datetime_start_time.send_keys(datetime.time(11, 55).__str__())
        datetime_stop_time.send_keys(datetime.time(12, 55).__str__())
        description.send_keys("This is a test work entered through the front-end")
        task = selenium.find_element_by_xpath('//*[@id="id_form-2-task"]/option[text()="'+self.test_1st_task.title+'"]')
        task.click()
        save_button = selenium.find_element_by_name('save')
        save_button.send_keys(Keys.RETURN)
        time.sleep(1)
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, 'id_form-1-project'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")
        assert_when_element_does_not_exist(self, '//*[@id="id_form-1-task"]')
        assert_when_element_exists(self, '//*[@id="id_form-2-task"]')
        work = Work.objects.get(description="This is a test work entered through the front-end")
        self.assertEqual(type(work), Work)
        work.delete()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-12 20:56
from __future__ import unicode_literals
from django.db import migrations


def reverse_func(apps, schema_editor):
    return 1


def backup_identifiers(apps, schema_editor):
    Position = apps.get_model("crm", "Position")
    CustomerGroupTransform = apps.get_model("crm", "CustomerGroupTransform")
    Price = apps.get_model("crm", "Price")
    UnitTransform = apps.get_model("crm", "UnitTransform")
    db_alias = schema_editor.connection.alias
    all_positions = Position.objects.using(db_alias).all()
    for position in all_positions:
        position.product_backup = position.product.id
        position.save()
    all_customer_group_transforms = CustomerGroupTransform.objects.using(db_alias).all()
    for customer_group_transform in all_customer_group_transforms:
        customer_group_transform.product_backup = customer_group_transform.product.id
        customer_group_transform.save()
    all_prices = Price.objects.using(db_alias).all()
    for price in all_prices:
        price.product_backup = price.product.id
        price.save()
    all_unit_transforms = UnitTransform.objects.using(db_alias).all()
    for unit_transform in all_unit_transforms:
        unit_transform.product_backup = unit_transform.product.id
        unit_transform.save()


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0048_auto_20181012_2056'),
    ]

    operations = [
        migrations.RunPython(backup_identifiers, reverse_func),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _


class CustomerGroupTransform(models.Model):
    from_customer_group = models.ForeignKey('CustomerGroup',
                                            on_delete=models.CASCADE,
                                            verbose_name=_("From Customer Group"),
                                            related_name="db_reltransfromfromcustomergroup",
                                            blank=False,
                                            null=False)
    to_customer_group = models.ForeignKey('CustomerGroup',
                                          on_delete=models.CASCADE,
                                          verbose_name=_("To Customer Group"),
                                          related_name="db_reltransfromtocustomergroup",
                                          blank=False,
                                          null=False)
    product_type = models.ForeignKey('ProductType',
                                     on_delete=models.CASCADE,
                                     verbose_name=_("Product Type"),
                                     blank=False,
                                     null=False)
    factor = models.DecimalField(verbose_name=_("Factor between From and To Customer Group"),
                                 blank=False,
                                 null=False,
                                 max_digits=17,
                                 decimal_places=2,)

    def transform(self, customer_group):
        """The transform function verifies whether the provided argument customer_group
        is corresponding with the "from_customer_group" variable of the CustomerGroupTransform class
        When this is ok, the function returns the "to_customer_group". When the provided customer_group
        argument is not corresponding, the function returns a "None"

        Args:
        customer_group: CustomerGroup object

        Returns:
        CustomerGroup object or None

        Raises:
        No exceptions planned"""
        if self.from_customer_group == customer_group:
            return self.to_customer_group
        else:
            return None

    def get_transform_factor(self):
        return self.factor

    def __str__(self):
        return "From " + self.from_customer_group.name + " to " + self.to_customer_group.name

    class Meta:
        app_label = "crm"
        verbose_name = _('Customer Group Price Transform')
        verbose_name_plural = _('Customer Group Price Transforms')


class CustomerGroupTransformInlineAdminView(admin.TabularInline):
    model = CustomerGroupTransform
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('', {
            'fields': ('from_customer_group',
                       'to_customer_group',
                       'factor',)
        }),
    )
    allow_add = True



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2019-01-02 22:30
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0055_auto_20181022_1937'),
    ]

    operations = [
        migrations.AlterField(
            model_name='position',
            name='product_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product'),
        ),
    ]



# -*- coding: utf-8 -*-

import datetime

import pytz


def limit_string_length(input_string, maximum_length):
    if len(input_string) > maximum_length:
        output_string = input_string[:maximum_length]
    else:
        output_string = input_string
    return output_string


def get_string_between(input_string, search_start, search_end):
    pos_start = input_string.index(search_start)
    pos_end = input_string.index(search_end)
    output_string = input_string[pos_start:pos_end]
    return output_string


def xstr(s):
    if s is None:
        return ""
    else:
        return str(s)


def get_today_date():
    return datetime.date.today()


def get_today_datetime():
    return datetime.datetime.today()


class ConditionalMethodDecorator(object):
    def __init__(self, dec, condition):
        self.decorator = dec
        self.condition = condition

    def __call__(self, func):
        if not self.condition:
            # Return the function unchanged, not decorated.
            return func
        return self.decorator(func)


def make_date_utc(input_date):
    output_date = pytz.timezone("UTC").localize(input_date, is_dst=None)
    return output_date



# -*- coding: utf-8 -*-

from django.contrib import admin

from koalixcrm.accounting.accounting.accounting_period import AccountingPeriod, OptionAccountingPeriod
from koalixcrm.accounting.accounting.booking import Booking, OptionBooking
from koalixcrm.accounting.accounting.account import Account, OptionAccount
from koalixcrm.accounting.accounting.product_category import ProductCategory, OptionProductCategory

admin.site.register(Account, OptionAccount)
admin.site.register(Booking, OptionBooking)
admin.site.register(ProductCategory, OptionProductCategory)
admin.site.register(AccountingPeriod, OptionAccountingPeriod)



import datetime
from django.test import TestCase
from django.contrib.auth.models import User
from koalixcrm.accounting.models import Account
from koalixcrm.accounting.models import AccountingPeriod
from koalixcrm.accounting.models import Booking
from koalixcrm.crm.documents.pdf_export import PDFExport
from koalixcrm.global_support_functions import make_date_utc


class AccountingModelTest(TestCase):
    def setUp(self):
        user = User.objects.create(username='Username',
                                   password="Userone")

        cash = Account.objects.create(account_number="1000",
                                      title="Cash",
                                      account_type="A",
                                      description="Highest liquid asset",
                                      is_open_interest_account=False,
                                      is_open_reliabilities_account=False,
                                      is_product_inventory_activa=False,
                                      is_a_customer_payment_account=True)
        bank_account = Account.objects.create(account_number="1300",
                                              title="Bank Account",
                                              account_type="A",
                                              description="Moderate liquid asset",
                                              is_open_interest_account=False,
                                              is_open_reliabilities_account=False,
                                              is_product_inventory_activa=False,
                                              is_a_customer_payment_account=True)
        bank_loan = Account.objects.create(account_number="2000",
                                      title="Shortterm bankloans",
                                      account_type="L",
                                      description="Shortterm loan",
                                      is_open_interest_account=False,
                                      is_open_reliabilities_account=False,
                                      is_product_inventory_activa=False,
                                      is_a_customer_payment_account=False)
        investment_capital = Account.objects.create(account_number="2900",
                                      title="Investment capital",
                                      account_type="L",
                                      description="Very longterm  loan",
                                      is_open_interest_account=False,
                                      is_open_reliabilities_account=False,
                                      is_product_inventory_activa=False,
                                      is_a_customer_payment_account=False)
        spendings = Account.objects.create(account_number="3000",
                                           title="Spendings",
                                           account_type="S",
                                           description="Purchase spendings",
                                           is_open_interest_account=False,
                                           is_open_reliabilities_account=False,
                                           is_product_inventory_activa=False,
                                           is_a_customer_payment_account=False)
        earnings = Account.objects.create(account_number="4000",
                                          title="Earnings",
                                          account_type="E",
                                          description="Sales account",
                                          is_open_interest_account=False,
                                          is_open_reliabilities_account=False,
                                          is_product_inventory_activa=False,
                                          is_a_customer_payment_account=False)
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        from_date = datetime_now.date()
        to_date = (datetime_now + datetime.timedelta(days=365)).date()
        accounting_period_2024 = AccountingPeriod.objects.create(title="Fiscal year 2024",
                                                                 begin=from_date,
                                                                 end=to_date)
        from_date = (datetime_now + datetime.timedelta(days=(365+1))).date()
        to_date = (datetime_now + datetime.timedelta(days=(365*2))).date()
        accounting_period_2025 = AccountingPeriod.objects.create(title="Fiscal year 2025",
                                        begin=from_date,
                                        end=to_date)
        from_date = (datetime_now + datetime.timedelta(days=(365*2+1))).date()
        to_date = (datetime_now + datetime.timedelta(days=(365*3))).date()
        AccountingPeriod.objects.create(title="Fiscal year 2026",
                                        begin=from_date,
                                        end=to_date)
        Booking.objects.create(from_account=cash,
                               to_account=spendings,
                               amount="1000",
                               description="This is the first booking",
                               booking_date=make_date_utc(datetime.datetime(2025, 1, 1, 0, 00)),
                               accounting_period=accounting_period_2025,
                               staff=user,
                               last_modified_by=user)

        Booking.objects.create(from_account=earnings,
                               to_account=cash,
                               amount="500",
                               description="This is the first booking",
                               booking_date=make_date_utc(datetime.datetime(2025, 1, 1, 0, 00)),
                               accounting_period=accounting_period_2025,
                               staff=user,
                               last_modified_by=user)

        Booking.objects.create(from_account=bank_loan,
                               to_account=cash,
                               amount="5000",
                               description="This is the first booking",
                               booking_date=make_date_utc(datetime.datetime(2025, 1, 1, 0, 00)),
                               accounting_period=accounting_period_2025,
                               staff=user,
                               last_modified_by=user)

        Booking.objects.create(from_account=investment_capital,
                               to_account=bank_account,
                               amount="490000",
                               description="This is the first booking",
                               booking_date=make_date_utc(datetime.datetime(2025, 1, 1, 0, 00)),
                               accounting_period=accounting_period_2024,
                               staff=user,
                               last_modified_by=user)

    def test_sumOfAllBookings(self):
        cash_account = Account.objects.get(title="Cash")
        spendings_account = Account.objects.get(title="Spendings")
        earnings_account = Account.objects.get(title="Earnings")
        self.assertEqual((cash_account.sum_of_all_bookings()).__str__(), "4500.00")
        self.assertEqual((spendings_account.sum_of_all_bookings()).__str__(), "1000.00")
        self.assertEqual((earnings_account.sum_of_all_bookings()).__str__(), "500.00")

    def test_sumOfAllBookingsBeforeAccountPeriod(self):
        cash_account = Account.objects.get(title="Cash")
        spendings_account = Account.objects.get(title="Spendings")
        earnings_account = Account.objects.get(title="Earnings")
        accounting_period_2026 = AccountingPeriod.objects.get(title="Fiscal year 2026")
        self.assertEqual((cash_account.sum_of_all_bookings_before_accounting_period(accounting_period_2026)).__str__(), "4500.00")
        self.assertEqual((spendings_account.sum_of_all_bookings_before_accounting_period(accounting_period_2026)).__str__(), "1000.00")
        self.assertEqual((earnings_account.sum_of_all_bookings_before_accounting_period(accounting_period_2026)).__str__(), "500.00")

    def test_sumOfAllBookingsWithinAccountgPeriod(self):
        cash_account = Account.objects.get(title="Cash")
        spendings_account = Account.objects.get(title="Spendings")
        earnings_account = Account.objects.get(title="Earnings")
        accounting_period_2024 = AccountingPeriod.objects.get(title="Fiscal year 2024")
        self.assertEqual((cash_account.sum_of_all_bookings_within_accounting_period(accounting_period_2024)).__str__(), "0")
        self.assertEqual((spendings_account.sum_of_all_bookings_within_accounting_period(accounting_period_2024)).__str__(), "0")
        self.assertEqual((earnings_account.sum_of_all_bookings_within_accounting_period(accounting_period_2024)).__str__(), "0")

    def test_overall_liabilities(self):
        accounting_period_2025 = AccountingPeriod.objects.get(title="Fiscal year 2025")
        self.assertEqual(
            (accounting_period_2025.overall_liabilities()).__str__(), "495000.00")

    def test_overall_assets(self):
        accounting_period_2025 = AccountingPeriod.objects.get(title="Fiscal year 2025")
        self.assertEqual(
            (accounting_period_2025.overall_assets()).__str__(), "494500.00")

    def test_overall_earnings(self):
        accounting_period_2025 = AccountingPeriod.objects.get(title="Fiscal year 2025")
        self.assertEqual(
            (accounting_period_2025.overall_earnings()).__str__(), "500.00")

    def test_overall_spendings(self):
        accounting_period_2025 = AccountingPeriod.objects.get(title="Fiscal year 2025")
        self.assertEqual(
            (accounting_period_2025.overall_spendings()).__str__(), "1000.00")

    def test_serialize_to_xml(self):
        accounting_period_2025 = AccountingPeriod.objects.get(title="Fiscal year 2025")
        xml = accounting_period_2025.serialize_to_xml()
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.account']/field[@name='title']", 'Earnings')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.account']/field[@name='title']", 'Spendings')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.account']/field[@name='title']", 'Investment capital')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.account']/field[@name='title']", 'Shortterm bankloans')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.account']/field[@name='title']",'Cash')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.account']/field[@name='title']",'Bank Account')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.accountingperiod']/Overall_Spendings", '1000.00')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.accountingperiod']/Overall_Earnings", '500.00')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.accountingperiod']/Overall_Assets", '494500.00')
        self.assertEqual(result, 1)
        result=PDFExport.find_element_in_xml(xml, "object/[@model='accounting.accountingperiod']/Overall_Liabilities", '495000.00')
        self.assertEqual(result, 1)



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import Product


class StandardProductFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Product
        django_get_or_create = ('product_number',)



# -*- coding: utf-8 -*-

from datetime import *
from django.db import models
from django.contrib import admin, messages
from django.utils.translation import ugettext as _
from koalixcrm.crm.const.purpose import *
from koalixcrm.global_support_functions import xstr, make_date_utc
from koalixcrm.crm.contact.phone_address import PhoneAddress
from koalixcrm.crm.contact.email_address import EmailAddress
from koalixcrm.crm.contact.postal_address import PostalAddress
from koalixcrm.crm.documents.sales_document_position import SalesDocumentPosition, SalesDocumentInlinePosition
from koalixcrm.djangoUserExtension.models import TextParagraphInDocumentTemplate, UserExtension
from koalixcrm.crm.product.product_type import ProductType
from koalixcrm.crm.exceptions import TemplateSetMissingInContract
import koalixcrm.crm.documents.calculations
from koalixcrm.crm.documents.pdf_export import PDFExport


class TextParagraphInSalesDocument(models.Model):
    sales_document = models.ForeignKey("SalesDocument", on_delete=models.CASCADE)
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=2, choices=PURPOSESTEXTPARAGRAPHINDOCUMENTS)
    text_paragraph = models.TextField(verbose_name=_("Text"), blank=False, null=False)

    def create_paragraph(self, default_paragraph, sales_document):
        self.sales_document = sales_document
        self.purpose = default_paragraph.purpose
        self.text_paragraph = default_paragraph.text_paragraph
        self.save()
        return

    class Meta:
        app_label = "crm"
        verbose_name = _('Text Paragraph In Sales Document')
        verbose_name_plural = _('Text Paragraphs In Sales Documents')

    def __str__(self):
        return self.id.__str__()


class SalesDocument(models.Model):
    contract = models.ForeignKey("Contract",
                                 on_delete=models.CASCADE,
                                 verbose_name=_('Contract'))
    external_reference = models.CharField(verbose_name=_("External Reference"),
                                          max_length=100,
                                          blank=True)
    discount = models.DecimalField(max_digits=5,
                                   decimal_places=2,
                                   verbose_name=_("Discount"),
                                   blank=True,
                                   null=True)
    description = models.CharField(verbose_name=_("Description"),
                                   max_length=100,
                                   blank=True,
                                   null=True)
    last_pricing_date = models.DateField(verbose_name=_("Pricing Date"),
                                         blank=True,
                                         null=True)
    last_calculated_price = models.DecimalField(max_digits=17,
                                                decimal_places=2,
                                                verbose_name=_("Price without Tax "),
                                                blank=True,
                                                null=True)
    last_calculated_tax = models.DecimalField(max_digits=17,
                                              decimal_places=2,
                                              verbose_name=_("Tax"),
                                              blank=True,
                                              null=True)
    customer = models.ForeignKey("Customer",
                                 on_delete=models.CASCADE,
                                 verbose_name=_("Customer"))
    staff = models.ForeignKey('auth.User',
                              on_delete=models.CASCADE,
                              limit_choices_to={'is_staff': True},
                              blank=True,
                              verbose_name=_("Staff"),
                              related_name="db_relscstaff",
                              null=True)
    currency = models.ForeignKey("Currency", on_delete=models.CASCADE, verbose_name=_("Currency"),
                                 blank=False, null=False)
    date_of_creation = models.DateTimeField(verbose_name=_("Created at"),
                                            auto_now_add=True)
    custom_date_field = models.DateField(verbose_name=_("Custom Date"),
                                         blank=True,
                                         null=True)
    last_modification = models.DateTimeField(verbose_name=_("Last modified"),
                                             auto_now=True)
    last_modified_by = models.ForeignKey('auth.User', on_delete=models.CASCADE, limit_choices_to={'is_staff': True},
                                         verbose_name=_("Last modified by"),
                                         related_name="db_lstscmodified",
                                         null=True,
                                         blank="True")
    template_set = models.ForeignKey("djangoUserExtension.DocumentTemplate",
                                     on_delete=models.CASCADE,
                                     verbose_name=_("Referred Template"),
                                     null=True,
                                     blank=True)
    derived_from_sales_document = models.ForeignKey("SalesDocument",
                                                    on_delete=models.CASCADE,
                                                    blank=True,
                                                    null=True)
    last_print_date = models.DateTimeField(verbose_name=_("Last printed"),
                                           blank=True,
                                           null=True)

    class Meta:
        app_label = "crm"
        verbose_name = _('Sales Document')
        verbose_name_plural = _('Sales Documents')

    def serialize_to_xml(self):
        from koalixcrm.crm.models import PostalAddressForContact
        from koalixcrm.crm.models import Currency
        from koalixcrm.crm.models import PurchaseOrder
        from koalixcrm.crm.models import SalesDocument
        from koalixcrm.crm.models import SalesDocumentPosition
        from koalixcrm.crm.models import Contact
        from django.contrib import auth
        objects = [self, ]
        position_class = SalesDocumentPosition
        objects += list(SalesDocument.objects.filter(id=self.id))
        if isinstance(self, PurchaseOrder):
            objects += list(Contact.objects.filter(id=self.supplier.id))
            objects += list(PostalAddressForContact.objects.filter(person=self.supplier.id))
            for address in list(PostalAddressForContact.objects.filter(person=self.supplier.id)):
                objects += list(PostalAddress.objects.filter(id=address.id))
        else:
            objects += list(Contact.objects.filter(id=self.customer.id))
            objects += list(PostalAddressForContact.objects.filter(person=self.customer.id))
            for address in list(PostalAddressForContact.objects.filter(person=self.customer.id)):
                objects += list(PostalAddress.objects.filter(id=address.id))
        objects += list(TextParagraphInSalesDocument.objects.filter(sales_document=self.id))
        objects += list(Currency.objects.filter(id=self.currency.id))
        objects += SalesDocumentPosition.add_positions(position_class, self)
        objects += list(auth.models.User.objects.filter(id=self.staff.id))
        objects += UserExtension.objects_to_serialize(self, self.staff)
        main_xml = PDFExport.write_xml(objects)
        return main_xml

    def is_complete_with_price(self):
        """ Checks whether the SalesContract is completed with a price, in case the
        SalesContract was not completed or the price calculation was not performed,
        the method returns false"""

        if self.last_pricing_date and self.last_calculated_price:
            return True
        else:
            return False

    def create_sales_document(self, calling_model):
        self.staff = calling_model.staff
        if isinstance(calling_model, koalixcrm.crm.documents.contract.Contract):
            self.contract = calling_model
            self.customer = calling_model.default_customer
            self.currency = calling_model.default_currency
            self.description = calling_model.description
            self.discount = 0
        elif isinstance(calling_model, SalesDocument):
            self.derived_from_sales_document = calling_model
            self.contract = calling_model.contract
            self.customer = calling_model.customer
            self.currency = calling_model.currency
            self.description = calling_model.description
            self.discount = calling_model.discount

    def attach_text_paragraphs(self):
        default_paragraphs = TextParagraphInDocumentTemplate.objects.filter(document_template=self.template_set)
        for default_paragraph in list(default_paragraphs):
            invoice_paragraph = TextParagraphInSalesDocument()
            invoice_paragraph.create_paragraph(default_paragraph, self)

    def attach_sales_document_positions(self, calling_model):
        if isinstance(calling_model, SalesDocument):
            sales_document_positions = SalesDocumentPosition.objects.filter(sales_document=calling_model.id)
            for sales_document_position in list(sales_document_positions):
                new_position = SalesDocumentPosition()
                new_position.create_position(sales_document_position, self)

    def create_pdf(self, template_set, printed_by):
        self.last_print_date = make_date_utc(datetime.now())
        self.save()
        return koalixcrm.crm.documents.pdf_export.PDFExport.create_pdf(self, template_set, printed_by)

    def get_template_set(self):
        if self.template_set:
            return self.template_set
        else:
            raise TemplateSetMissingInContract((_("Template Set missing in Sales Document" + str(self))))

    def get_fop_config_file(self, template_set):
        template_set = self.get_template_set()
        return template_set.get_fop_config_file()

    def get_xsl_file(self, template_set):
        template_set = self.get_template_set()
        return template_set.get_xsl_file()

    def __str__(self):
        return _("Sales Contract") + ": " + str(self.id) + " " + _("from Contract") + ": " + str(self.contract.id)


class PostalAddressForSalesDocument(PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    sales_document = models.ForeignKey("SalesDocument", on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Postal Address For Sales Documents')
        verbose_name_plural = _('Postal Address For Sales Documents')

    def __str__(self):
        return xstr(self.pre_name) + ' ' + xstr(self.name) + ' ' + xstr(self.address_line_1)


class EmailAddressForSalesDocument(EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    sales_document = models.ForeignKey("SalesDocument", on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address For Sales Documents')
        verbose_name_plural = _('Email Address For Sales Documents')

    def __str__(self):
        return str(self.email)


class PhoneAddressForSalesDocument(PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    sales_document = models.ForeignKey("SalesDocument", on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address For Sales Documents')
        verbose_name_plural = _('Phone Address For Sales Documents')

    def __str__(self):
        return str(self.phone)


class SalesDocumentTextParagraph(admin.StackedInline):
    model = TextParagraphInSalesDocument
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('text_paragraph', 'purpose', )
        }),
    )
    allow_add = True


class SalesDocumentPostalAddress(admin.StackedInline):
    model = PostalAddressForSalesDocument
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('prefix',
                       'pre_name',
                       'name',
                       'address_line_1',
                       'address_line_2',
                       'address_line_3',
                       'address_line_4',
                       'zip_code',
                       'town',
                       'state',
                       'country',
                       'purpose')
        }),
    )
    allow_add = True


class SalesDocumentPhoneAddress(admin.TabularInline):
    model = PhoneAddressForSalesDocument
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('phone', 'purpose',)
        }),
    )
    allow_add = True


class SalesDocumentEmailAddress(admin.TabularInline):
    model = EmailAddressForSalesDocument
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('email', 'purpose',)
        }),
    )
    allow_add = True


class OptionSalesDocument(admin.ModelAdmin):
    list_display = ('id',
                    'description',
                    'contract',
                    'customer',
                    'currency',
                    'staff',
                    'last_modified_by',
                    'last_calculated_price',
                    'last_calculated_tax',
                    'last_pricing_date',
                    'last_modification',
                    'last_print_date')
    list_display_links = ('id',)
    list_filter = ('customer',
                   'contract',
                   'currency',
                   'staff',
                   'last_modification')
    ordering = ('-id',)
    search_fields = ('contract__id',
                     'customer__name',
                     'currency__description')

    fieldsets = (
        (_('Sales Contract'), {
            'fields': ('contract',
                       'description',
                       'customer',
                       'currency',
                       'discount',
                       'staff',
                       'external_reference',
                       'template_set',
                       'custom_date_field')
        }),
    )
    save_as = True
    inlines = [SalesDocumentInlinePosition, SalesDocumentTextParagraph,
               SalesDocumentPostalAddress, SalesDocumentPhoneAddress,
               SalesDocumentEmailAddress]

    def response_add(self, request, obj, post_url_continue=None):
        new_obj = self.after_saving_model_and_related_inlines(request, obj)
        new_obj.custom_date_field = date.today().__str__()
        return super(OptionSalesDocument, self).response_add(request=request,
                                                             obj=new_obj,
                                                             post_url_continue=post_url_continue)

    def response_change(self, request, new_object):
        obj = self.after_saving_model_and_related_inlines(request, new_object)
        return super(OptionSalesDocument, self).response_change(request, obj)

    def after_saving_model_and_related_inlines(self, request, obj):
        try:
            koalixcrm.crm.documents.calculations.Calculations.calculate_document_price(obj, date.today())
            self.message_user(request, "Successfully calculated Prices")
        except (ProductType.NoPriceFound, SalesDocumentPosition.NoPriceFound) as e:
            self.message_user(request, "Unsuccessful in updating the Prices " + e.__str__(), level=messages.ERROR)
        return obj

    def save_model(self, request, obj, form, change):
        if change:
            obj.last_modified_by = request.user
        else:
            obj.last_modified_by = request.user
            obj.staff = request.user
        obj.save()

    def create_quote(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.quote.Quote,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_quote.short_description = _("Create Quote")

    def create_invoice(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.invoice.Invoice,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_invoice.short_description = _("Create Invoice")

    def create_purchase_confirmation(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.purchase_confirmation.PurchaseConfirmation,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_purchase_confirmation.short_description = _("Create Purchase Confirmation")

    def create_delivery_note(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.delivery_note.DeliveryNote,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_delivery_note.short_description = _("Create Delivery note")

    def create_payment_reminder(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.payment_reminder.PaymentReminder,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_payment_reminder.short_description = _("Create Payment Reminder")

    def create_purchase_order(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.purchase_order.PurchaseOrder,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_purchase_order.short_description = _("Create Purchase Order")

    def create_pdf(self, request, queryset):
        from koalixcrm.crm.views.pdfexport import PDFExportView
        for obj in queryset:
            response = PDFExportView.export_pdf(self,
                                                request,
                                                obj,
                                                ("/admin/crm/"+obj.__class__.__name__.lower()+"/"),
                                                obj.template_set)
            return response

    create_pdf.short_description = _("Create PDF")

    def create_project(self, request, queryset):
        from koalixcrm.crm.views.create_task import CreateTaskView
        for obj in queryset:
            response = CreateTaskView.create_project(self,
                                                     request,
                                                     obj,
                                                     ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_project.short_description = _("Create Project")



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-13 22:28
from __future__ import unicode_literals

from django.db import migrations, models
from filebrowser.fields import FileBrowseField
import django.db.models.deletion


def fill_subscription_type_from_backup(apps, schema_editor):
    Subscription = apps.get_model("subscriptions", "Subscription")
    SubscriptionType = apps.get_model("subscriptions", "SubscriptionType")
    SubscriptionTypeBackup = apps.get_model("subscriptions", "SubscriptionTypeBackup")
    ProductType = apps.get_model("crm", "ProductType")
    db_alias = schema_editor.connection.alias
    all_subscription_type_backups = SubscriptionTypeBackup.objects.using(db_alias).all()
    for subscription_type_backup in all_subscription_type_backups:
        product_type = ProductType.objects.using(db_alias).create(
            description=subscription_type_backup.description,
            title=subscription_type_backup.title,
            default_unit=subscription_type_backup.default_unit,
            date_of_creation=subscription_type_backup.date_of_creation,
            last_modification=subscription_type_backup.last_modification,
            last_modified_by=subscription_type_backup.last_modified_by,
            tax=subscription_type_backup.tax)
        product_type.save()

        subscription_type = SubscriptionType.objects.using(db_alias).create(
            product_type=product_type,
            contract_document=subscription_type_backup.contract_document,
            payment_interval=subscription_type_backup.payment_interval,
            minimum_duration=subscription_type_backup.minimum_duration,
            automatic_contract_extension_reminder=subscription_type_backup.automatic_contract_extension_reminder,
            automatic_contract_extension=subscription_type_backup.automatic_contract_extension,
            cancellation_period=subscription_type_backup.cancellation_period)
        subscription_type.save()

        all_subscriptions = Subscription.objects.using(db_alias).filter(
            subscription_type_backup=subscription_type_backup.old_id)
        for subscription in all_subscriptions:
            subscription.subscription_type=subscription_type
            subscription.save()

def reverse_func(apps, schema_editor):
    return 1


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0053_auto_20181014_2305'),
        ('subscriptions', '0004_auto_20181013_2213'),
    ]

    operations = [
        migrations.CreateModel(
            name='SubscriptionType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('cancellation_period', models.IntegerField(verbose_name="Cancellation Period (months)",
                                                            blank=True,
                                                            null=True)),
                ('automatic_contract_extension', models.IntegerField(
                    verbose_name="Automatic Contract Extension (months)",
                    blank=True,
                    null=True)),
                ('automatic_contract_extension_reminder', models.IntegerField(
                    verbose_name="Automatic Contract Extension Reminder (days)",
                    blank=True,
                    null=True)),
                ('minimum_duration', models.IntegerField(verbose_name="Minimum Contract Duration",
                                                         blank=True,
                                                         null=True)),
                ('payment_interval', models.IntegerField(verbose_name="Payment Interval (days)",
                                                         blank=True,
                                                         null=True)),
                ('contract_document', FileBrowseField(verbose_name="Contract Documents",
                                                      blank=True,
                                                      null=True,
                                                      max_length=200)),
                ('product_type', models.ForeignKey('crm.ProductType',
                                                   verbose_name='Product Type',
                                                   on_delete=models.deletion.SET_NULL,
                                                   null=True,
                                                   blank=True)),
             ],
            options={
                'verbose_name': 'Subscription Type',
                'verbose_name_plural': 'Subscription Types',
            },
        ),
        migrations.AddField(
            model_name='subscription',
            name='subscription_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    to='SubscriptionType',
                                    verbose_name='Subscription Type'),
            preserve_default=False,
        ),
        migrations.RunPython(fill_subscription_type_from_backup, reverse_func),
        migrations.AlterField(
            model_name='subscription',
            name='subscription_type',
            field=models.ForeignKey(null=True,
                                    on_delete=django.db.models.deletion.CASCADE,
                                    to='subscriptions.SubscriptionType',
                                    verbose_name='Subscription Type'),

        ),
        migrations.DeleteModel('SubscriptionTypeBackup'),
        migrations.RemoveField(
            model_name='subscription',
            name='subscription_type_backup',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-09 19:49
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import filebrowser.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='templateset',
            options={'verbose_name': 'Vorlagenset', 'verbose_name_plural': 'Vorlagensets'},
        ),
        migrations.AlterModelOptions(
            name='userextension',
            options={'verbose_name': 'Benutzer Erweiterung', 'verbose_name_plural': 'Benutzer Erweiterungen'},
        ),
        migrations.AlterModelOptions(
            name='xslfile',
            options={'verbose_name': 'XSL Datei', 'verbose_name_plural': 'XSL Dateien'},
        ),
        migrations.AlterField(
            model_name='templateset',
            name='addresser',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Absender'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='balancesheetXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltemplatebalancesheet', to='djangoUserExtension.XSLFile', verbose_name='XSL Datei für die Billanz'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='bankingaccountref',
            field=models.CharField(blank=True, max_length=60, null=True, verbose_name='Bankverbindung'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='deilveryorderXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltemplatedeliveryorder', to='djangoUserExtension.XSLFile', verbose_name='XSL Datei für den Lieferschein'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='footerTextpurchaseorders',
            field=models.TextField(blank=True, null=True, verbose_name='Grussfromel für die Kaufaufträge'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='footerTextsalesorders',
            field=models.TextField(blank=True, null=True, verbose_name='Grussfromel für die Verkaufsaufträge'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='fopConfigurationFile',
            field=filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True, verbose_name='FOP Configurations Datei'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='headerTextpurchaseorders',
            field=models.TextField(blank=True, null=True, verbose_name='Begrüssungsformel für die Kaufaufträge'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='headerTextsalesorders',
            field=models.TextField(blank=True, null=True, verbose_name='Begrüssungsfromel für die Verkaufsaufträge'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='invoiceXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltemplateinvoice', to='djangoUserExtension.XSLFile', verbose_name='XSL Datei für die Rechnung'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='logo',
            field=filebrowser.fields.FileBrowseField(blank=True, max_length=200, null=True, verbose_name="Logo für die PDF's"),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='organisationname',
            field=models.CharField(max_length=200, verbose_name='Name der Organisation'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='pagefooterleft',
            field=models.CharField(blank=True, max_length=40, null=True, verbose_name='Fussnote Links'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='pagefootermiddle',
            field=models.CharField(blank=True, max_length=40, null=True, verbose_name='Fussnote Mitte'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='profitLossStatementXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltemplateprofitlossstatement', to='djangoUserExtension.XSLFile', verbose_name='XSL Datei für die Erfolgsrechung'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='purchaseorderXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltemplatepurchaseorder', to='djangoUserExtension.XSLFile', verbose_name='XSL Datei für die Kaufsbestätigung'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='quoteXSLFile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltemplatequote', to='djangoUserExtension.XSLFile', verbose_name='XSL Datei für die Offerte'),
        ),
        migrations.AlterField(
            model_name='templateset',
            name='title',
            field=models.CharField(max_length=100, verbose_name='Titel'),
        ),
        migrations.AlterField(
            model_name='userextensionemailaddress',
            name='purpose',
            field=models.CharField(choices=[('H', 'Privat'), ('O', 'Geschäftlich'), ('P', 'Mobil Privat'), ('B', 'Mobil Geschäftlich')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='userextensionphoneaddress',
            name='purpose',
            field=models.CharField(choices=[('H', 'Privat'), ('O', 'Geschäftlich'), ('P', 'Mobil Privat'), ('B', 'Mobil Geschäftlich')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='userextensionpostaladdress',
            name='purpose',
            field=models.CharField(choices=[('H', 'Privat'), ('O', 'Geschäftlich'), ('P', 'Mobil Privat'), ('B', 'Mobil Geschäftlich')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='xslfile',
            name='title',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Titel'),
        ),
        migrations.AlterField(
            model_name='xslfile',
            name='xslfile',
            field=filebrowser.fields.FileBrowseField(max_length=200, verbose_name='XSL Datei'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Provides XML rendering support.
"""

import os
from subprocess import check_output, STDOUT
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from rest_framework.renderers import TemplateHTMLRenderer
from rest_framework_xml.renderers import XMLRenderer


class XSLFORenderer(TemplateHTMLRenderer):
    """
    Renderer which serializes to PDF using XSLT and FO
    """
    media_type_unused = 'application/pdf'
    media_type = 'plain/text'
    format = 'pdf'
    charset = 'utf-8'

    def get_template_names(self, response, view):
        """Override with xslt_names. This allows for you
        to still render HTML if you want
        """
        if hasattr(response, 'xslt_name'):
            return [response.latex_name]
        elif self.xslt_name:
            return [self.xslt_name]
        elif hasattr(view, 'get_xslt_names'):
            return view.get_xslt_names()
        elif hasattr(view, 'xslt_name'):
            return [view.xslt_name]
        raise ImproperlyConfigured(
            u'Returned a template response with no `xslt_name` attribute '
            u'set on either the view or response')

    def perform_xsl_transformation(self, file_with_serialized_xml, xsl_file, fop_config_file, file_output_pdf):
        check_output([settings.FOP_EXECUTABLE,
                      '-c', fop_config_file.path_full,
                      '-xml', os.path.join(settings.PDF_OUTPUT_ROOT, file_with_serialized_xml),
                      '-xsl', xsl_file.path_full,
                      '-pdf', file_output_pdf], stderr=STDOUT)

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        First renders `data` into serialized XML, afterwards start fop and return PDF
        """
        if data is None:
            return ''

        super(XSLFORenderer, self).render(data, accepted_media_type, renderer_context)

        xml_renderer = XMLRenderer()
        xml_string = xml_renderer.render(data, accepted_media_type, renderer_context)
        xml_file = open(renderer_context['view'].file_name, "wb+")
        xml_file.truncate()
        xml_file.write(xml_string.encode('utf-8'))
        xml_file.close()
        # perform xsl transformation
        self.perform_xsl_transformation(xml_file,
                                        self.xsl_file,
                                        self.fop_config_file,
                                        self.file_output_pdf)

        # Read file
        with open(self.file_output_pdf, 'rb') as f:
            rendered_output = f.read()

        return rendered_output


from rest_framework import serializers

from koalixcrm.crm.contact.customer_group import CustomerGroup


class OptionCustomerGroupJSONSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(required=False)
    name = serializers.CharField(read_only=True)

    class Meta:
        model = CustomerGroup
        fields = ('id',
                  'name')


class CustomerGroupJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = CustomerGroup
        fields = ('id',
                  'name')



import datetime
import pytest
from django.test import TestCase
from koalixcrm.crm.factories.factory_customer_billing_cycle import StandardCustomerBillingCycleFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_work import StandardWorkFactory
from koalixcrm.crm.factories.factory_task_status import DoneTaskStatusFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.crm.factories.factory_human_resource import StandardHumanResourceFactory
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.global_support_functions import make_date_utc
from koalixcrm.crm.factories.factory_estimation import StandardHumanResourceEstimationToTaskFactory


class TaskEffectiveDuration(TestCase):

    def setUp(self):
        datetime_now = make_date_utc(datetime.datetime.now())
        start_first_task = (datetime_now - datetime.timedelta(days=30)).date()
        start_second_task = (datetime_now - datetime.timedelta(days=60)).date()
        end_date_first_task = (datetime_now + datetime.timedelta(days=30)).date()
        end_date_second_task = (datetime_now + datetime.timedelta(days=60)).date()

        self.test_billing_cycle = StandardCustomerBillingCycleFactory.create()
        self.test_customer_group = StandardCustomerGroupFactory.create()
        self.test_customer = StandardCustomerFactory.create(is_member_of=(self.test_customer_group,))
        self.test_currency = StandardCurrencyFactory.create()
        self.human_resource = StandardHumanResourceFactory.create()
        self.test_reporting_period = StandardReportingPeriodFactory.create()
        self.test_1st_task = StandardTaskFactory.create(title="1st Test Task",
                                                        project=self.test_reporting_period.project)
        self.estimation_1st_task = StandardHumanResourceEstimationToTaskFactory.create(task=self.test_1st_task,
                                                                                       date_from=start_first_task,
                                                                                       date_until=end_date_first_task)
        self.test_2nd_task = StandardTaskFactory.create(title="2nd Test Task",
                                                        project=self.test_reporting_period.project)
        self.estimation_2nd_task = StandardHumanResourceEstimationToTaskFactory.create(task=self.test_2nd_task,
                                                                                       date_from=start_second_task,
                                                                                       date_until=end_date_second_task)

    @pytest.mark.back_end_tests
    def test_effective_duration(self):
        datetime_now = make_date_utc(datetime.datetime.now())
        datetime_later_2 = datetime_now+datetime.timedelta(days=1)
        datetime_later_4 = datetime_now+datetime.timedelta(days=3)
        self.assertEqual(
            (self.test_1st_task.planned_duration()).__str__(), "60")
        self.assertEqual(
            (self.test_1st_task.effective_duration()).__str__(), "Task has not yet ended")
        self.assertEqual(
            (self.test_2nd_task.planned_duration()).__str__(), "120")
        self.assertEqual(
            (self.test_2nd_task.effective_duration()).__str__(), "Task has not yet ended")

        new_status = DoneTaskStatusFactory.create()
        self.test_1st_task.status = new_status
        self.test_1st_task.save()
        self.test_2nd_task.status = new_status
        self.test_2nd_task.save()

        self.assertEqual(
            (self.test_1st_task.planned_duration()).__str__(), "60")
        self.assertEqual(
            (self.test_1st_task.effective_duration()).__str__(), "30")
        self.assertEqual(
            (self.test_2nd_task.planned_duration()).__str__(), "120")
        self.assertEqual(
            (self.test_2nd_task.effective_duration()).__str__(), "60")

        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=datetime_now.date(),
            worked_hours="1.00",
            task=self.test_1st_task,
            reporting_period=self.test_reporting_period
        )
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=datetime_later_2.date(),
            worked_hours="1.00",
            task=self.test_1st_task,
            reporting_period=self.test_reporting_period
        )
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=datetime_now.date(),
            worked_hours="1.00",
            task=self.test_2nd_task,
            reporting_period=self.test_reporting_period
        )
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=datetime_later_4.date(),
            worked_hours="1.00",
            task=self.test_2nd_task,
            reporting_period=self.test_reporting_period
        )

        self.assertEqual(
            (self.test_1st_task.planned_duration()).__str__(), "60")
        self.assertEqual(
            (self.test_1st_task.effective_duration()).__str__(), "1")
        self.assertEqual(
            (self.test_2nd_task.planned_duration()).__str__(), "120")
        self.assertEqual(
            (self.test_2nd_task.effective_duration()).__str__(), "3")



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import Unit


class StandardUnitFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Unit
        django_get_or_create = ('short_name',)

    description = "Kilogram"
    short_name = "kg"
    is_a_fraction_of = None
    fraction_factor_to_next_higher_unit = None


class SmallUnitFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Unit
        django_get_or_create = ('short_name',)

    description = "Gram"
    short_name = "g"
    is_a_fraction_of = factory.SubFactory(StandardUnitFactory)
    fraction_factor_to_next_higher_unit = 1000



from rest_framework import serializers

from koalixcrm.djangoUserExtension.user_extension.user_extension import UserExtension
from koalixcrm.crm.reporting.resource_manager import ResourceManager
from koalixcrm.djangoUserExtension.rest.user_extension_rest import OptionUserExtensionJSONSerializer


class OptionResourceManagerJSONSerializer(serializers.HyperlinkedModelSerializer):
    user = OptionUserExtensionJSONSerializer(source='user',
                                             read_only=True)

    class Meta:
        model = ResourceManager
        fields = ('user',)


class ResourceManagerJSONSerializer(serializers.HyperlinkedModelSerializer):
    user = OptionUserExtensionJSONSerializer(source='user')

    class Meta:
        model = ResourceManager
        fields = ('user',)

    def create(self, validated_data):
        resource_manager = ResourceManager()
        # Deserialize user
        user = validated_data.pop('user')
        if user:
            if user.get('id', None):
                resource_manager.user = UserExtension.objects.get(id=user.get('id', None))
            else:
                resource_manager.user = None

    def update(self, resource_manager, validated_data):
        # Deserialize user
        user = validated_data.pop('user')
        if user:
            if user.get('id', resource_manager.resource_manager):
                resource_manager.user = UserExtension.objects.get(id=user.get('id', None))
            else:
                resource_manager.user = resource_manager.user_id
        else:
            resource_manager.user = None



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import TaskLinkType


class RelatedToTaskLinkTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TaskLinkType
        django_get_or_create = ('title',)

    title = "Is related to"
    description = "This task is related with ...."


class RequiresLinkTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TaskLinkType
        django_get_or_create = ('title',)

    title = "This task requires"
    description = "This task requires the completion or the existence of ...."



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import CurrencyTransform
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory, SecondStandardCurrencyFactory
from koalixcrm.crm.factories.factory_product_type import StandardProductTypeFactory


class StandardCurrencyTransformFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = CurrencyTransform
        django_get_or_create = ('from_currency',
                                'to_currency')

    from_currency = factory.SubFactory(StandardCurrencyFactory)
    to_currency = factory.SubFactory(SecondStandardCurrencyFactory)
    product_type = factory.SubFactory(StandardProductTypeFactory)
    factor = 0.90



# -*- coding: utf-8 -*-

from decimal import *
from django.db import models
from django.utils.translation import ugettext as _
from django.contrib import admin
from django.utils.html import format_html
from koalixcrm.crm.reporting.agreement import Agreement
from koalixcrm.crm.reporting.agreement import AgreementInlineAdminView
from koalixcrm.crm.reporting.estimation import EstimationInlineAdminView
from koalixcrm.crm.reporting.generic_task_link import InlineGenericTaskLink
from koalixcrm.crm.reporting.work import WorkInlineAdminView, Work
from koalixcrm.crm.reporting.reporting_period import ReportingPeriod
from koalixcrm.crm.reporting.resource_price import ResourcePrice
from koalixcrm.crm.reporting.estimation import Estimation
from koalixcrm.crm.documents.pdf_export import PDFExport
from koalixcrm.crm.exceptions import ReportingPeriodNotFound
from rest_framework import serializers
from koalixcrm import global_support_functions


class Task(models.Model):
    """ The Task model"""
    title = models.CharField(verbose_name=_("Title"),
                             max_length=100,
                             blank=True,
                             null=True)
    project = models.ForeignKey("Project",
                                on_delete=models.CASCADE,
                                verbose_name=_('Project'),
                                related_name='tasks',
                                blank=False,
                                null=False)
    description = models.TextField(verbose_name=_("Description"),
                                   blank=True,
                                   null=True)
    status = models.ForeignKey("TaskStatus", on_delete=models.CASCADE, verbose_name=_('Status'),
                               blank=True,
                               null=True)
    last_status_change = models.DateField(verbose_name=_("Last Status Change"),
                                          blank=True,
                                          null=False)
    previous_status = None

    def __init__(self, *args, **kwargs):
        super(Task, self).__init__(*args, **kwargs)
        self.previous_status = self.status

    def save(self, *args, **kwargs):
        if self.id is not None:
            if self.status != self.previous_status:
                self.last_status_change = global_support_functions.get_today_date()
        elif self.last_status_change is None:
            self.last_status_change = global_support_functions.get_today_date()
        super(Task, self).save(*args, **kwargs)

    def link_to_task(self):
        if self.id:
            return format_html("<a href='/admin/crm/task/%s' >%s</a>" % (str(self.id), str(self.title)))
        else:
            return "Not present"
    link_to_task.short_description = _("Task")

    def planned_duration(self):
        if (not self.planned_start()) or (not self.planned_end()):
            duration_in_days = "n/a"
        elif self.planned_start() > self.planned_end():
            duration_in_days = "n/a"
        else:
            duration_in_days = (self.planned_end()-self.planned_start()).days
        return duration_in_days
    planned_duration.short_description = _("Planned Duration [dys]")
    planned_duration.tags = True

    def planned_start(self):
        """ The function return the planned start of a task as a date based on the estimations which are
         attached to the task in case there was no estimation attached to this task, the function returns None

         Args:
         no arguments

         Returns:
         planned_start (Date) or None

         Raises:
         No exceptions planned"""
        all_task_estimations = Estimation.objects.filter(task=self.id)
        planned_task_start = None
        if len(all_task_estimations) == 0:
            planned_task_start = None
        else:
            for estimation in all_task_estimations:
                if planned_task_start is None:
                    planned_task_start = estimation.date_from
                elif estimation.date_from < planned_task_start:
                    planned_task_start = estimation.date_from
        return planned_task_start
    planned_start.short_description = _("Planned Start")
    planned_start.tags = True

    def planned_end(self):
        """ The function return the planned end of a task as a date based on the estimations which are
         attached to the task in case there was no estimation attached to this task, the function returns None

         Args:
         no arguments

         Returns:
         planned_end (Date) or None

         Raises:
         No exceptions planned"""
        all_task_estimations = Estimation.objects.filter(task=self.id)
        planned_task_end = None
        if len(all_task_estimations) == 0:
            planned_task_end = None
        else:
            for estimation in all_task_estimations:
                if not planned_task_end:
                    planned_task_end = estimation.date_until
                elif estimation.date_until < planned_task_end:
                    planned_task_end = estimation.date_until
        return planned_task_end
    planned_end.short_description = _("Planned End")
    planned_end.tags = True

    def planned_effort(self, reporting_period=None, remaining=False):
        """The function return the planned effort of resources which have been estimated for this task
        at a specific reporting period. When no reporting_period is provided, the last reporting period
        is selected

        Args:
        remaining (Boolean)

        Returns:
        planned effort (Decimal) [hrs], 0 if when no estimation are present

        Raises:
        no exceptions expected"""
        latest_estimation = self.get_latest_estimation()
        effort = 0
        if latest_estimation:
            try:
                predecessor_reporting_period = latest_estimation.reporting_period.get_predecessor(latest_estimation.reporting_period,
                                                                                                  latest_estimation.reporting_period.project)
            except ReportingPeriodNotFound:
                effort = 0
                predecessor_reporting_period = None
            if remaining:
                effort = 0
            else:
                while predecessor_reporting_period:
                    effort += self.effective_effort(reporting_period=predecessor_reporting_period)
                    try:
                        predecessor_reporting_period = predecessor_reporting_period.get_predecessor(predecessor_reporting_period,
                                                                                                    predecessor_reporting_period.project)
                    except ReportingPeriodNotFound:
                        predecessor_reporting_period = None
            effort += latest_estimation.amount
        else:
            effort = 0
        return effort
    planned_effort.short_description = _("Planned Effort")
    planned_effort.tags = True

    def get_latest_estimation(self):
        estimations = Estimation.objects.filter(task=self.id)
        latest_estimation = None
        for estimation in estimations:
            if not latest_estimation:
                latest_estimation = estimation
            else:
                if latest_estimation.reporting_period.end < estimation.reporting_period.begin:
                    latest_estimation = estimation
        return latest_estimation

    def planned_costs_in_buckets(self, reporting_period=None, buckets=None):
        """The function returns the planned costs of resources which have been estimated for this task
         at a specific reporting period plus the costs of the effective effort before the provided reporting_period
         When no reporting_period is provided. The costs are split into the provided buckets.
         The passed argument reporting_period is used to specify the root of the estimation.

        Following example shall illustrate what to expect from this function:
        The project started 01.01.2018,
        1st reporting period ranged from 01.01.2018 to 31.01.2018
        2nd reporting period ranged from 01.02.2018 to 28.02.2018
        3rd reporting period range from 01.03.2018 to 31.03.2018
        There are not further reporting periods defined
        At the beginning of second reporting period an estimation was done. The estimation reached from
        Beginning of 01.02.2018 until 31.12.2018 and the total cost was 100kUSD.
        The function should be called this way: task.planned_costs_in_buckets(2nd ReportingPeriod,
                                                                              list(2nd ReportingPeriod,
                                                                                   3rd ReportingPeriod,))
        the function will return a dict containing the accumulated planned costs for the list of provided
        buckets and an additional bucked containing the overall sum (also containing the parts of the
        estimations which did not fit into provided reporting periods)

        Args:
        bucket (list of ReportingPeriods)
        reporting_period (ReportingPeriod)

        Returns:
        planned costs (Decimal), 0 if when no estimation or no reporting period is attached to the task

        Raises:
        No exceptions planned"""
        planned_costs = dict()

        latest_estimation = self.get_latest_estimation()
        if buckets:
            for bucket in buckets:
                planned_costs[bucket] = 0
        planned_costs['sum_costs'] = 0
        if latest_estimation:
            if buckets:
                for bucket in buckets:
                    if bucket.end < latest_estimation.reporting_period.begin:
                        planned_costs[bucket] += planned_costs['sum_costs']
                        planned_costs[bucket] += self.effective_costs(reporting_period=bucket)
                        planned_costs['sum_costs'] = planned_costs[bucket]
                    else:
                        planned_costs[bucket] += planned_costs['sum_costs']
                        planned_costs[bucket] += latest_estimation.calculated_costs(bucket_start=bucket.begin,
                                                                                    bucket_end=bucket.end)
                        planned_costs['sum_costs'] = planned_costs[bucket]
        return planned_costs

    def planned_costs(self, reporting_period=None, remaining=False):
        """The function returns the planned overall costs of resources which have been estimated for this task
         at a specific reporting period plus the costs of the effective effort before the provided reporting_period
         When no reporting_period is provided, the last reporting period is selected.

        Args:
        no arguments

        Returns:
        planned costs (Decimal), 0 if when no estimation or no reporting period is present

        Raises:
        No exceptions planned"""
        planned_costs = 0
        if reporting_period:
            planned_costs_in_buckets = self.planned_costs_in_buckets(reporting_period=reporting_period, buckets=None)
            for key in planned_costs_in_buckets.keys():
                planned_costs += planned_costs_in_buckets[key]
        else:
            planned_effort = self.planned_effort(remaining=remaining)
            latest_estimation = self.get_latest_estimation()
            if latest_estimation:
                resource_prices = ResourcePrice.objects.filter(resource=latest_estimation.resource)
                price = 0
                if len(resource_prices) != 0:
                    for resource_price in resource_prices:
                        price = resource_price.price
                        break
                planned_costs = planned_effort*price
            else:
                planned_costs = 0
        return planned_costs
    planned_costs.short_description = _("Planned Costs")
    planned_costs.tags = True

    def planned_total_costs(self):
        return self.planned_costs(remaining=False)
    planned_total_costs.short_description = _("Planned Total Costs")
    planned_total_costs.tags = True

    def effective_start(self):
        """The function return the effective start of a task as a date. The
        function return the effective start of a task as a date based on the reported work
        in case there was no work reported to this task, the planned start is used as a
        fall-back.

        Args:
        no arguments

        Returns:
        effective_start (Date)

        Raises:
        No exceptions planned"""
        all_task_works = Work.objects.filter(task=self.id)
        effective_task_start = None
        if len(all_task_works) == 0:
            effective_task_start = self.planned_start()
        else:
            for work in all_task_works:
                if not effective_task_start:
                    effective_task_start = work.date
                elif work.date < effective_task_start:
                    effective_task_start = work.date
        return effective_task_start
    effective_start.short_description = _("Effective Start")
    effective_start.tags = True

    def task_end(self):
        """The function returns a boolean value True when the task is on
        status done. In all other case (example in no status case) the function
        returns False

        Args:
        no arguments

        Returns:
        task_ended (Boolean)

        Raises:
        No exceptions planned"""
        if not self.status:
            task_ended = False
        elif self.status.is_done:
            task_ended = self.status.is_done
        else:
            task_ended = False
        return task_ended

    def effective_end(self):
        """When the task has already ended, the
        function return the effective end of a task as a date based on the reported work
        in case there was no work reported to this task, the last status change is used as a
        fall-back. When the task did not yet end, the function returns None

        Args:
        no arguments

        Returns:
        effective_end (Date) or None when not yet ended

        Raises:
        No exceptions planned"""
        all_task_works = Work.objects.filter(task=self.id)
        effective_task_end = None
        if self.task_end():
            if len(all_task_works) == 0:
                effective_task_end = self.last_status_change
            else:
                for work in all_task_works:
                    if not effective_task_end:
                        effective_task_end = work.date
                    elif work.date > effective_task_end:
                        effective_task_end = work.date
        else:
            effective_task_end = None
        return effective_task_end
    effective_end.short_description = _("Effective End")
    effective_end.tags = True

    def effective_duration(self):
        """The function return the effective overall duration of a task as a string in days
        The function is reading the effective_starts and effective_ends of the task and
        subtract them from each other.

        Args:
        no arguments

        Returns:
        duration [dys]

        Raises:
        No exceptions planned"""
        effective_end = self.effective_end()
        effective_start = self.effective_start()
        if not effective_start:
            duration_as_string = "Task has not yet started"
        elif not effective_end:
            duration_as_string = "Task has not yet ended"
        else:
            duration_as_date = self.effective_end()-self.effective_start()
            duration_as_string = duration_as_date.days.__str__()
        return duration_as_string
    effective_duration.short_description = _("Effective Duration [dys]")
    effective_duration.tags = True

    def serialize_to_xml(self, reporting_period):
        objects = [self, ]
        main_xml = PDFExport.write_xml(objects)
        if reporting_period:
            works = Work.objects.filter(task=self.id,
                                        reporting_period=reporting_period)
        else:
            works = Work.objects.filter(task=self.id)
        for work in works:
            work_xml = work.serialize_to_xml()
            main_xml = PDFExport.merge_xml(main_xml, work_xml)
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.task']",
                                                       "Effective_Costs_Confirmed_Overall",
                                                       self.effective_costs_confirmed())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.task']",
                                                       "Effective_Costs_Not_Confirmed_Overall",
                                                       self.effective_costs_not_confirmed())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.task']",
                                                       "Effective_Effort_Overall",
                                                       self.effective_effort(reporting_period=None))
        if reporting_period:
            main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                           "object/[@model='crm.task']",
                                                           "Effective_Costs_InPeriod",
                                                           self.effective_costs(reporting_period=reporting_period))
            main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                           "object/[@model='crm.task']",
                                                           "Effective_Effort_InPeriod",
                                                           self.effective_effort(reporting_period=reporting_period))
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.task']",
                                                       "Planned_Effort",
                                                       self.planned_costs())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.task']",
                                                       "Effective_Duration",
                                                       self.effective_duration())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.task']",
                                                       "Planned_Duration",
                                                       self.planned_duration())
        return main_xml

    def effective_effort_overall(self):
        return self.effective_effort(reporting_period=None)
    effective_effort_overall.short_description = _("Effective Effort [hrs]")
    effective_effort_overall.tags = True

    def effective_effort(self, reporting_period=None):
        """ Effective effort returns the effective effort on a task
        when reporting_period is None, the effective effort overall is calculated
        when reporting_period is specified, the effective effort in this period is calculated"""
        if reporting_period:
            work_objects = Work.objects.filter(task=self.id,
                                               reporting_period=reporting_period)
        else:
            work_objects = Work.objects.filter(task=self.id)
        sum_effort = 0
        for work_object in work_objects:
            sum_effort += work_object.effort_seconds()
        sum_effort_in_hours = sum_effort / 3600
        return Decimal(sum_effort_in_hours)

    def effective_costs(self, reporting_period=None, confirmed=True):
        """Returns the effective costs on a task
        when reporting_period is None, the effective costs overall is calculated
        when reporting_period is specified, the effective effort in this period is calculated
        the costs are calculated in the project currency

        Args:
          no arguments

        Returns:
          costs [Currency] (Decimal)

        Raises:
           no exception planned
        """
        agreements = Agreement.objects.filter(task=self)
        if reporting_period:
            all_work_in_task = Work.objects.filter(task=self.id,
                                                   reporting_period=reporting_period)
        elif not confirmed:
            all_work_in_task = Work.objects.filter(task=self.id,
                                                   reporting_period__status__is_done=False)
        else:
            all_work_in_task = Work.objects.filter(task=self.id,
                                                   reporting_period__status__is_done=True)
        sum_costs = Decimal(0)
        work_with_agreement = list()
        work_without_agreement = list()
        work_calculated = list()
        human_resource_list = dict()
        for work_object in all_work_in_task:
            if work_object.human_resource not in human_resource_list:
                human_resource_list[work_object.human_resource] = dict()
            for agreement in agreements:
                agreement_matches = agreement.match_with_work(work_object)
                if agreement_matches:
                    if work_object not in work_with_agreement:
                        work_with_agreement.append(work_object)
                    if agreement not in human_resource_list.get(work_object.human_resource):
                        human_resource_list.get(work_object.human_resource)[agreement] = list()
                    human_resource_list.get(work_object.human_resource).get(agreement).append(work_object)
            if work_object not in work_with_agreement:
                work_without_agreement.append(work_object)

        for human_resource_dict in human_resource_list:
            agreement_list = Agreement.objects.filter(task=self, resource=human_resource_dict).order_by('costs__price')
            for agreement in agreement_list:
                agreement_remaining_amount = agreement.amount
                if human_resource_list[human_resource_dict].get(agreement):
                    for work in human_resource_list[human_resource_dict].get(agreement):
                        if work in work_with_agreement:
                            if (agreement_remaining_amount - work.worked_hours) > 0:
                                agreement_remaining_amount -= work.worked_hours
                                sum_costs += Decimal(work.effort_hours())*agreement.costs.price
                                work_calculated.append(work)
        for work in all_work_in_task:
            if work not in work_calculated:
                if work not in work_without_agreement:
                    work_without_agreement.append(work)
        for work in work_without_agreement:
            default_resource_prices = ResourcePrice.objects.filter(resource=work.human_resource.id).order_by('price')
            if default_resource_prices:
                default_resource_price = default_resource_prices[0]
                sum_costs += Decimal(work.effort_hours())*default_resource_price.price
            else:
                sum_costs = Decimal(0)
                break
        sum_costs = self.project.default_currency.round(sum_costs)
        return sum_costs

    def effective_costs_confirmed(self):
        return self.effective_costs()

    effective_costs_confirmed.short_description = _("Effective costs confirmed")
    effective_costs_confirmed.tags = True

    def effective_costs_not_confirmed(self):
        return self.effective_costs(confirmed=False)

    effective_costs_not_confirmed.short_description = _("Effective costs not confirmed")
    effective_costs_not_confirmed.tags = True

    def is_reporting_allowed(self):
        """Returns True when the task is available for reporting,
        Returns False when the task is not available for reporting,
        The decision whether the task is available for reporting is purely depending
        on the task_status. When the status is done or when the status is unknown,
        the task is not longer available for reporting.

        Args:
          no arguments

        Returns:
          allowed (Boolean)

        Raises:
           when there is no valid reporting Period"""
        if self.status:
            if self.status.is_done:
                allowed = False
            else:
                allowed = True
        else:
            allowed = False
        return allowed
    is_reporting_allowed.short_description = _("Reporting")
    is_reporting_allowed.tags = True

    def get_title(self):
        if self.title:
            return self.title
        else:
            return "n/a"

    def __str__(self):
        return str(self.id) + " " + self.get_title()

    class Meta:
        app_label = "crm"
        verbose_name = _('Task')
        verbose_name_plural = _('Tasks')


class TaskAdminView(admin.ModelAdmin):
    list_display = ('link_to_task',
                    'planned_start',
                    'planned_end',
                    'project',
                    'status',
                    'last_status_change',
                    'planned_duration',
                    'planned_total_costs',
                    'effective_duration',
                    'effective_effort_overall',
                    'effective_costs_confirmed',
                    'effective_costs_not_confirmed')
    list_display_links = ('link_to_task',)
    list_filter = ('project',)
    ordering = ('-id',)

    fieldsets = (
        (_('Work'), {
            'fields': ('title',
                       'project',
                       'description',
                       'status')
        }),
    )
    save_as = True
    inlines = [AgreementInlineAdminView,
               EstimationInlineAdminView,
               InlineGenericTaskLink,
               WorkInlineAdminView]


class TaskInlineAdminView(admin.TabularInline):
    model = Task
    readonly_fields = ('link_to_task',
                       'last_status_change',
                       'planned_start',
                       'planned_end',
                       'planned_duration',
                       'planned_total_costs',
                       'effective_duration',
                       'effective_effort_overall',
                       'effective_costs_confirmed',
                       'effective_costs_not_confirmed')
    fieldsets = (
        (_('Task'), {
            'fields': ('link_to_task',
                       'title',
                       'planned_start',
                       'planned_end',
                       'status',
                       'last_status_change',
                       'planned_duration',
                       'planned_total_costs',
                       'effective_duration',
                       'effective_effort_overall',
                       'effective_costs_confirmed',
                       'effective_costs_not_confirmed')
        }),
    )
    extra = 1

    def has_add_permission(self, request):
        return True

    def has_delete_permission(self, request, obj=None):
        return False



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _


class EmailAddress(models.Model):
    email = models.EmailField(max_length=200,
                              verbose_name=_("Email Address"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address')
        verbose_name_plural = _('Email Address')



# -*- coding: utf-8 -*-

from datetime import *
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from django.utils.html import format_html
from koalixcrm.crm.const.status import *
from koalixcrm.plugin import *
from koalixcrm.crm.documents.sales_document import SalesDocument, OptionSalesDocument
from koalixcrm.global_support_functions import limit_string_length


class Quote(SalesDocument):
    valid_until = models.DateField(verbose_name=_("Valid until"))
    status = models.CharField(max_length=1, choices=QUOTESTATUS, verbose_name=_('Status'))

    def link_to_quote(self):
        if self.id:
            return format_html("<a href='/admin/crm/quote/%s' >%s</a>" % (str(self.id),
                                                                          limit_string_length(str(self.description),
                                                                                              30)))
        else:
            return "Not present"
    link_to_quote.short_description = _("Quote");

    def create_from_reference(self, calling_model):
        self.create_sales_document(calling_model)
        self.status = 'I'
        self.valid_until = date.today().__str__()
        self.date_of_creation = date.today().__str__()
        self.template_set = self.contract.get_template_set(self)
        self.save()
        self.attach_sales_document_positions(calling_model)
        self.attach_text_paragraphs()

    def __str__(self):
        return _("Quote") + ": " + self.id.__str__() + " " + _("from Contract") + ": " + self.contract.id.__str__()

    class Meta:
        app_label = "crm"
        verbose_name = _('Quote')
        verbose_name_plural = _('Quotes')


class OptionQuote(OptionSalesDocument):
    list_display = OptionSalesDocument.list_display + ('valid_until',
                                                       'status',)
    list_filter = OptionSalesDocument.list_filter + ('status',)
    ordering = OptionSalesDocument.ordering
    search_fields = OptionSalesDocument.search_fields
    fieldsets = OptionSalesDocument.fieldsets + (
        (_('Quote specific'), {
            'fields': ('valid_until',
                       'status', )
        }),
    )

    save_as = OptionSalesDocument.save_as
    inlines = OptionSalesDocument.inlines

    actions = ['create_purchase_confirmation',
               'create_invoice',
               'create_quote',
               'create_delivery_note',
               'create_purchase_order',
               'create_project',
               'create_pdf']

    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("quoteInlines"))


class InlineQuote(admin.TabularInline):
    model = Quote
    classes = ['collapse']
    show_change_link = True
    can_delete = True
    extra = 1
    readonly_fields = ('link_to_quote',
                       'contract',
                       'customer',
                       'valid_until',
                       'status',
                       'last_pricing_date',
                       'last_calculated_price',
                       'last_calculated_tax',)
    fieldsets = (
        (_('Quote'), {
            'fields': ('link_to_quote',
                       'contract',
                       'customer',
                       'valid_until',
                       'status',
                       'last_pricing_date',
                       'last_calculated_price',
                       'last_calculated_tax',)
        }),
    )
    allow_add = False



# -*- coding: utf-8 -*-

from django.utils.translation import ugettext as _
from koalixcrm.crm.documents.sales_document import SalesDocument, OptionSalesDocument


class PurchaseConfirmation(SalesDocument):

    def create_from_reference(self, calling_model):
        self.create_sales_document(calling_model)
        self.template_set = self.contract.get_template_set(self)
        self.save()
        self.attach_sales_document_positions(calling_model)
        self.attach_text_paragraphs()

    def __str__(self):
        return _("Purchase Confirmation") + ": " + self.id.__str__() + " " + _("from Contract") + ": " + self.contract.id.__str__()

    class Meta:
        app_label = "crm"
        verbose_name = _('Purchase Confirmation')
        verbose_name_plural = _('Purchase Confirmations')


class OptionPurchaseConfirmation(OptionSalesDocument):
    list_display = OptionSalesDocument.list_display
    list_filter = OptionSalesDocument.list_filter
    ordering = OptionSalesDocument.ordering
    search_fields = OptionSalesDocument.search_fields
    fieldsets = OptionSalesDocument.fieldsets

    save_as = OptionSalesDocument.save_as
    inlines = OptionSalesDocument.inlines
    actions = ['create_invoice', 'create_quote',
               'create_delivery_note', 'create_purchase_order', 'create_', 'create_pdf']



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-04 21:52
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0010_auto_20180104_2138'),
    ]

    operations = [
        migrations.RenameField(
            model_name='currency',
            old_name='shortName',
            new_name='short_name',
        ),
        migrations.RenameField(
            model_name='customergrouptransform',
            old_name='fromCustomerGroup',
            new_name='from_customer_group',
        ),
        migrations.RenameField(
            model_name='customergrouptransform',
            old_name='toCustomerGroup',
            new_name='to_customer_group',
        ),
        migrations.RenameField(
            model_name='price',
            old_name='customerGroup',
            new_name='customer_group',
        ),
        migrations.RenameField(
            model_name='price',
            old_name='validfrom',
            new_name='valid_from',
        ),
        migrations.RenameField(
            model_name='price',
            old_name='validuntil',
            new_name='valid_until',
        ),
        migrations.RenameField(
            model_name='tax',
            old_name='accountActiva',
            new_name='account_activa',
        ),
        migrations.RenameField(
            model_name='tax',
            old_name='accountPassiva',
            new_name='account_passiva',
        ),
        migrations.RenameField(
            model_name='tax',
            old_name='taxrate',
            new_name='tax_rate',
        ),
        migrations.RenameField(
            model_name='unit',
            old_name='fractionFactorToNextHigherUnit',
            new_name='fraction_factor_to_next_higher_unit',
        ),
        migrations.RenameField(
            model_name='unit',
            old_name='isAFractionOf',
            new_name='is_a_fraction_of',
        ),
        migrations.RenameField(
            model_name='unit',
            old_name='shortName',
            new_name='short_name',
        ),
        migrations.RenameField(
            model_name='unittransform',
            old_name='fromUnit',
            new_name='from_unit',
        ),
        migrations.RenameField(
            model_name='unittransform',
            old_name='toUnit',
            new_name='to_unit',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-12-10 21:26
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0004_auto_20171210_2126'),
        ('crm', '0006_auto_20171210_1805'),
    ]

    operations = [
        migrations.CreateModel(
            name='TextParagraphInDocumentTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('purpose', models.CharField(choices=[('BS', 'Before subject'), ('AS', 'After subject'), ('BT', 'Before total'), ('AT', 'After total'), ('BW', 'Before wishes'), ('AW', 'After wishes'), ('C1', 'Custom 1'), ('C2', 'Custom 2'), ('C3', 'Custom 3'), ('C4', 'Custom 4')], max_length=2, verbose_name='Purpose')),
                ('text_paragraph', models.TextField(verbose_name='Text')),
                ('document_template', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'TextParagraphInDocumentTemplate',
                'verbose_name_plural': 'TextParagraphInDocumentTemplates',
            },
        ),
        migrations.AlterField(
            model_name='textparagraphinsalescontract',
            name='purpose',
            field=models.CharField(choices=[('BS', 'Before subject'), ('AS', 'After subject'), ('BT', 'Before total'), ('AT', 'After total'), ('BW', 'Before wishes'), ('AW', 'After wishes'), ('C1', 'Custom 1'), ('C2', 'Custom 2'), ('C3', 'Custom 3'), ('C4', 'Custom 4')], max_length=2, verbose_name='Purpose'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-12 20:01
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
        ('crm', '0033_auto_20180612_1936'),
    ]

    operations = [
        migrations.CreateModel(
            name='GenericProjectLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('date_of_creation', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('last_modified_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_project_link_last_modified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by')),
                ('project_link_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ProjectLinkType', verbose_name='Project Link Type')),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Project', verbose_name='Project')),
            ],
            options={
                'verbose_name': 'Project Link',
                'verbose_name_plural': 'Project Links',
            },
        ),
    ]



import pytest
import datetime
from django.test import TestCase
from koalixcrm.crm.documents.calculations import Calculations
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_quote import StandardQuoteFactory
from koalixcrm.crm.factories.factory_sales_document_position import StandardSalesDocumentPositionFactory
from koalixcrm.crm.factories.factory_product_type import StandardProductTypeFactory
from koalixcrm.crm.factories.factory_product_price import StandardPriceFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory, AdvancedCustomerGroupFactory
from koalixcrm.crm.factories.factory_tax import StandardTaxFactory
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory, SmallUnitFactory
from koalixcrm.global_support_functions import make_date_utc
from koalixcrm.crm.models import SalesDocumentPosition


class DocumentSalesDocumentPosition(TestCase):
    def setUp(self):
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        start_date = (datetime_now - datetime.timedelta(days=30)).date()
        end_date = (datetime_now + datetime.timedelta(days=30)).date()
        self.tax = StandardTaxFactory.create(tax_rate=10)
        self.test_currency_with_rounding = StandardCurrencyFactory.create(rounding=1)
        self.test_currency_without_rounding = StandardCurrencyFactory.create(
            rounding=None,
            description='Euro',
            short_name='EUR'
        )
        self.alternative_currency = self.test_currency_without_rounding
        self.customer_group = StandardCustomerGroupFactory.create()
        self.alternative_customer_group = AdvancedCustomerGroupFactory.create()
        self.customer = StandardCustomerFactory.create()
        self.customer.is_member_of.add(self.customer_group)
        self.customer.save()
        self.unit = StandardUnitFactory.create()
        self.alternative_unit = SmallUnitFactory.create()
        self.product_without_dates = StandardProductTypeFactory.create(
            product_type_identifier="A",
            tax=self.tax
        )
        self.price_without_customer_group = StandardPriceFactory.create(
            product_type=self.product_without_dates,
            customer_group=None,
            price=100,
            unit=self.unit,
            currency=self.test_currency_with_rounding,
            valid_from=start_date,
            valid_until=end_date
        )

    @pytest.mark.back_end_tests
    def test_calculate_document_price_overwritten(self):
        quote_1 = StandardQuoteFactory.create(customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            product_type=self.product_without_dates,
            overwrite_product_price=True,
            position_price_per_unit=90,
            unit=self.unit,
            sales_document=quote_1
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_1,
            pricing_date=date_now)
        self.assertEqual(
            quote_1.last_calculated_price.__str__(), "90.00")
        self.assertEqual(
            quote_1.last_calculated_tax.__str__(), "9.00")

    @pytest.mark.back_end_tests
    def test_calculate_document_price_overwritten_WithNone(self):
        quote_2 = StandardQuoteFactory.create(customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            product_type=self.product_without_dates,
            overwrite_product_price=True,
            position_price_per_unit=None,
            unit=self.unit,
            sales_document=quote_2
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        try:
            Calculations.calculate_document_price(
                document=quote_2,
                pricing_date=date_now)
        except SalesDocumentPosition.NoPriceFound as e:
            self.assertEqual(
                e.__str__(), "There is no Price set for the sales document position"
            )



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import TaskStatus


class DoneTaskStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TaskStatus
        django_get_or_create = ('title',)

    title = "Done"
    description = "This task is done"
    is_done = True


class PlannedTaskStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TaskStatus
        django_get_or_create = ('title',)

    title = "Planned"
    description = "This task is planned"
    is_done = False


class StartedTaskStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TaskStatus
        django_get_or_create = ('title',)

    title = "Started"
    description = "This task is started"
    is_done = False



# Generated by Django 2.1.5 on 2019-01-06 15:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0056_auto_20190102_2230'),
        ('crm', '0056_auto_20181216_2224'),
    ]

    operations = [
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.crm.product.price import Price


class ProductPrice(Price):
    product_type = models.ForeignKey("ProductType",
                                     on_delete=models.CASCADE,
                                     verbose_name=_("Product Type"))

    def __str__(self):
        return str(self.price) + " " + str(self.currency.short_name)

    class Meta:
        app_label = "crm"
        verbose_name = _('Product Price')
        verbose_name_plural = _('Product Prices')


class ProductPriceInlineAdminView(admin.TabularInline):
    model = ProductPrice
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('', {
            'fields': ('price',
                       'currency',
                       'unit',
                       'valid_from',
                       'valid_until',
                       'customer_group')
        }),
    )
    allow_add = True



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-12 20:56
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def reverse_func(apps, schema_editor):
    return 1


def restore_from_backup(apps, schema_editor):
    Position = apps.get_model("crm", "Position")
    ProductType = apps.get_model("crm", "ProductType")
    CustomerGroupTransform = apps.get_model("crm", "CustomerGroupTransform")
    Price = apps.get_model("crm", "Price")
    ProductPrice = apps.get_model("crm", "ProductPrice")
    UnitTransform = apps.get_model("crm", "UnitTransform")
    db_alias = schema_editor.connection.alias
    all_positions = Position.objects.using(db_alias).all()
    for position in all_positions:
        product_type = ProductType.objects.using(db_alias).get(id=position.product_backup)
        position.product_type = product_type
        position.save()
    all_customer_group_transforms = CustomerGroupTransform.objects.using(db_alias).all()
    for customer_group_transform in all_customer_group_transforms:
        product_type = ProductType.objects.using(db_alias).get(id=customer_group_transform.product_backup)
        customer_group_transform.product_type = product_type
        customer_group_transform.save()
    all_prices = Price.objects.using(db_alias).all()
    for price in all_prices:
        product_type = ProductType.objects.using(db_alias).get(id=price.product_backup)
        new_product_price = ProductPrice.objects.using(db_alias).create(unit=price.unit,
                                                                        currency=price.currency,
                                                                        customer_group=price.customer_group,
                                                                        price=price.price,
                                                                        valid_from=price.valid_from,
                                                                        valid_until=price.valid_until,
                                                                        product_type=product_type)
        new_product_price.save()
        price.delete()
    all_unit_transforms = UnitTransform.objects.using(db_alias).all()
    for unit_transform in all_unit_transforms:
        product_type = ProductType.objects.using(db_alias).get(id=unit_transform.product_backup)
        unit_transform.product_type = product_type
        unit_transform.save()


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0051_auto_20181014_2302'),
    ]

    operations = [
        migrations.RunPython(restore_from_backup, reverse_func),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-08-06 20:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0046_auto_20180805_2051'),
    ]

    operations = [
        migrations.AddField(
            model_name='work',
            name='worked_hours',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Worked Hours'),
        ),
    ]



# -*- coding: utf-8 -*-
from rest_framework import serializers

from koalixcrm.accounting.accounting.account import Account


class OptionAccountJSONSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(required=False)
    accountNumber = serializers.IntegerField(source='account_number', read_only=True)
    title = serializers.CharField(read_only=True)

    class Meta:
        model = Account
        fields = ('id',
                  'accountNumber',
                  'title')


class AccountJSONSerializer(serializers.HyperlinkedModelSerializer):
    accountNumber = serializers.IntegerField(source='account_number', allow_null=False)
    accountType = serializers.CharField(source='account_type', allow_null=False)
    isOpenReliabilitiesAccount = serializers.BooleanField(source='is_open_reliabilities_account')
    isOpenInterestAccount = serializers.BooleanField(source='is_open_interest_account')
    isProductInventoryActiva = serializers.BooleanField(source='is_product_inventory_activa')
    isCustomerPaymentAccount = serializers.BooleanField(source='is_a_customer_payment_account')

    class Meta:
        model = Account
        fields = ('id',
                  'accountNumber',
                  'title',
                  'accountType',
                  'description',
                  'isOpenReliabilitiesAccount',
                  'isOpenInterestAccount',
                  'isProductInventoryActiva',
                  'isCustomerPaymentAccount')
        depth = 1



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import Customer
from koalixcrm.crm.factories.factory_contact import StandardContactFactory
from koalixcrm.crm.factories.factory_customer_billing_cycle import StandardCustomerBillingCycleFactory


class StandardCustomerFactory(StandardContactFactory):
    class Meta:
        model = Customer

    default_customer_billing_cycle = factory.SubFactory(StandardCustomerBillingCycleFactory)
    is_lead = True

    @factory.post_generation
    def is_member_of(self, create, extracted):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of groups were passed in, use them
            for group in extracted:
                self.is_member_of.add(group)



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import Currency


class StandardCurrencyFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Currency
        django_get_or_create = ('description',)

    description = "Swiss Francs"
    short_name = "CHF"
    rounding = "0.05"


class SecondStandardCurrencyFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Currency
        django_get_or_create = ('description',)

    description = "Euro"
    short_name = "EUR"
    rounding = "0.01"



import datetime
import pytest
from django.test import TestCase
from koalixcrm.crm.factories.factory_user import AdminUserFactory
from koalixcrm.crm.factories.factory_customer_billing_cycle import StandardCustomerBillingCycleFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.djangoUserExtension.factories.factory_user_extension import StandardUserExtensionFactory
from koalixcrm.crm.reporting.task import Task


@pytest.fixture()
def freeze(monkeypatch):
    """ Now() manager patches date return a fixed, settable, value
        (freezes date)
    """
    original = datetime.date

    class FreezeMeta(type):
        def __instancecheck__(self, instance):
            if instance.isinstance(original) or instance.isinstance(Freeze):
                return True

    class Freeze(datetime.datetime):
        __metaclass__ = FreezeMeta

        @classmethod
        def freeze(cls, val):
            cls.frozen = val

        @classmethod
        def today(cls):
            return cls.frozen

        @classmethod
        def delta(cls, timedelta=None, **kwargs):
            """ Moves time fwd/bwd by the delta"""
            from datetime import timedelta as td
            if not timedelta:
                timedelta = td(**kwargs)
            cls.frozen += timedelta

    monkeypatch.setattr(datetime, 'date', Freeze)
    Freeze.freeze(original.today())
    return Freeze


class TaskConstructorTest(TestCase):

    @pytest.fixture(autouse=True)
    def freeze_time(self, freeze):
        self._freeze = freeze

    def setUp(self):

        self.test_billing_cycle = StandardCustomerBillingCycleFactory.create()
        self.test_user = AdminUserFactory.create()
        self.test_customer_group = StandardCustomerGroupFactory.create()
        self.test_customer = StandardCustomerFactory.create(is_member_of=(self.test_customer_group,))
        self.test_currency = StandardCurrencyFactory.create()
        self.test_user_extension = StandardUserExtensionFactory.create(user=self.test_user)
        self.test_reporting_period = StandardReportingPeriodFactory.create()

    def test_task_constructor(self):
        self._freeze.freeze(datetime.date(2024, 6, 2))
        task_minimal_1 = Task.objects.create(
            project=self.test_reporting_period.project,
        )
        task_minimal_1.save()
        self.assertEquals(task_minimal_1.last_status_change, datetime.date(2024, 6, 2))
        self._freeze.freeze(datetime.date(2018, 6, 15))
        task_minimal_2 = Task.objects.create(
            project=self.test_reporting_period.project,
        )
        task_minimal_2.save()
        self.assertEquals(task_minimal_2.last_status_change, datetime.date(2018, 6, 15))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-11-10 17:32
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('accounting', '0004_auto_20171009_1949'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'ordering': ['accountNumber'], 'verbose_name': 'Account', 'verbose_name_plural': 'Account'},
        ),
        migrations.AlterModelOptions(
            name='accountingperiod',
            options={'verbose_name': 'Accounting Period', 'verbose_name_plural': 'Accounting Periods'},
        ),
        migrations.AlterModelOptions(
            name='booking',
            options={'verbose_name': 'Booking', 'verbose_name_plural': 'Bookings'},
        ),
        migrations.AlterModelOptions(
            name='productcategorie',
            options={'verbose_name': 'Product Categorie', 'verbose_name_plural': 'Product Categories'},
        ),
        migrations.AlterField(
            model_name='account',
            name='accountNumber',
            field=models.IntegerField(verbose_name='Account Number'),
        ),
        migrations.AlterField(
            model_name='account',
            name='accountType',
            field=models.CharField(choices=[('E', 'Earnings'), ('S', 'Spendings'), ('L', 'Liabilities'), ('A', 'Assets')], max_length=1, verbose_name='Account Type'),
        ),
        migrations.AlterField(
            model_name='account',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='account',
            name='isACustomerPaymentAccount',
            field=models.BooleanField(verbose_name='Is a Customer Payment Account'),
        ),
        migrations.AlterField(
            model_name='account',
            name='isProductInventoryActiva',
            field=models.BooleanField(verbose_name='Is a Product Inventory Account'),
        ),
        migrations.AlterField(
            model_name='account',
            name='isopeninterestaccount',
            field=models.BooleanField(verbose_name='Is The Open Interests Account'),
        ),
        migrations.AlterField(
            model_name='account',
            name='isopenreliabilitiesaccount',
            field=models.BooleanField(verbose_name='Is The Open Liabilities Account'),
        ),
        migrations.AlterField(
            model_name='account',
            name='title',
            field=models.CharField(max_length=50, verbose_name='Account Title'),
        ),
        migrations.AlterField(
            model_name='accountingperiod',
            name='begin',
            field=models.DateField(verbose_name='Begin'),
        ),
        migrations.AlterField(
            model_name='accountingperiod',
            name='end',
            field=models.DateField(verbose_name='End'),
        ),
        migrations.AlterField(
            model_name='accountingperiod',
            name='title',
            field=models.CharField(max_length=200, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='accountingPeriod',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='accounting.AccountingPeriod', verbose_name='AccountingPeriod'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='amount',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Amount'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='bookingDate',
            field=models.DateTimeField(verbose_name='Booking at'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='bookingReference',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Invoice', verbose_name='Booking Reference'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='dateofcreation',
            field=models.DateTimeField(auto_now=True, verbose_name='Created at'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='description',
            field=models.CharField(blank=True, max_length=120, null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='fromAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_fromaccount', to='accounting.Account', verbose_name='From Account'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='lastmodification',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Last modified'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='lastmodifiedby',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_lstmodified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='staff',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_refstaff', to=settings.AUTH_USER_MODEL, verbose_name='Reference Staff'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='toAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_toaccount', to='accounting.Account', verbose_name='To Account'),
        ),
        migrations.AlterField(
            model_name='productcategorie',
            name='lossAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_loss_account', to='accounting.Account', verbose_name='Loss Account'),
        ),
        migrations.AlterField(
            model_name='productcategorie',
            name='profitAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_profit_account', to='accounting.Account', verbose_name='Profit Account'),
        ),
        migrations.AlterField(
            model_name='productcategorie',
            name='title',
            field=models.CharField(max_length=50, verbose_name='Product Categorie Title'),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
from django.http import Http404
from django.shortcuts import render
from django.contrib.admin.widgets import *
from django.contrib.admin import helpers
from django.template.context_processors import csrf
from koalixcrm.djangoUserExtension.exceptions import TooManyUserExtensionsAvailable
from koalixcrm.crm.views.pdfexport import PDFExportView


class CreateWorkReportForm(forms.Form):
    range_from = forms.DateField(widget=AdminDateWidget)
    range_to = forms.DateField(widget=AdminDateWidget)


def create_work_report(calling_model_admin, request, queryset):
    try:
        if request.POST.get('post'):
            if 'create_pdf' in request.POST:
                create_work_report_form = CreateWorkReportForm(request.POST)
                if create_work_report_form.is_valid():
                    for obj in queryset:
                        return PDFExportView.export_pdf(
                            calling_model_admin=calling_model_admin,
                            request=request,
                            source=obj,
                            redirect_to="/admin/",
                            template_to_use=obj.user.default_template_set.work_report_template,
                            date_from=create_work_report_form.cleaned_data["range_from"],
                            date_to=create_work_report_form.cleaned_data["range_to"])
        else:
            datetime_now = datetime.datetime.today()
            range_from_date = (datetime_now - datetime.timedelta(days=30)).date()
            range_to_date = datetime_now.date()
            create_work_report_form = CreateWorkReportForm(initial={'range_from': range_from_date,
                                                                    'range_to': range_to_date})
            c = {'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
                 'queryset': queryset,
                 'form': create_work_report_form}
            c.update(csrf(request))
            return render(request, 'crm/admin/create_work_report.html', c)
    except TooManyUserExtensionsAvailable:
        raise Http404





# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-27 16:19
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0043_reportingperiodstatus'),
    ]

    operations = [
        migrations.AddField(
            model_name='reportingperiod',
            name='status',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ReportingPeriodStatus', verbose_name='Reporting Period Status'),
        ),
    ]



# -*- coding: utf-8 -*-

from os import path

from koalixcrm import crm
from koalixcrm import djangoUserExtension
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand
from filebrowser.base import FileObject
from django.utils.translation import ugettext as _
from django.conf import settings

DEFAULT_FILE = 'dashboard.py'


class Command(BaseCommand):
    help = (
    'This Command is going to install the default Templates, given by the koalixcrm base installation, in your django instance. Be sure you first run syncdb')
    args = '[]'
    label = 'application name'

    @staticmethod
    def store_default_template_xsl_file(language, file_name):
        file_path = Command.path_of_default_template_file(language, file_name)
        xsl_file = Command.store_xsl_file(file_path)
        return xsl_file

    @staticmethod
    def path_of_default_template_file(language, file_name):
        file_path = path.join(settings.STATIC_ROOT, "default_templates", language, file_name)
        f = None;
        try:
            f = open(file_path,'r')
        except (FileNotFoundError) as e:
            print(_("File not found:") + file_path)
            print(_("Run collectstatic command and fix potential errors"))
        finally:
            if f is not None:
                f.close()
        return file_path

    @staticmethod
    def store_xsl_file(xsl_file_path):
        xsl_file = djangoUserExtension.models.XSLFile()
        xsl_file.title = path.basename(xsl_file_path)
        xsl_file.xslfile = FileObject(xsl_file_path)
        xsl_file.save()
        return xsl_file

    def handle(self, *args, **options):
        template_set = djangoUserExtension.models.TemplateSet()
        template_set.title = 'default_template_set'
        template_set.invoiceXSLFile = Command.store_default_template_xsl_file("en", "invoice.xsl")
        template_set.quoteXSLFile = Command.store_default_template_xsl_file("en", "quote.xsl")
        template_set.purchaseconfirmationXSLFile = Command.store_default_template_xsl_file("en", "purchaseconfirmation.xsl")
        template_set.purchaseorderXSLFile = Command.store_default_template_xsl_file("en", "purchaseorder.xsl")
        template_set.deilveryorderXSLFile = Command.store_default_template_xsl_file("en", "deliveryorder.xsl")

        if 'koalixcrm.accounting' in settings.INSTALLED_APPS:
            template_set.profitLossStatementXSLFile = Command.store_default_template_xsl_file("en", "profitlossstatement.xsl")
            template_set.balancesheetXSLFile = Command.store_default_template_xsl_file("en", "balancesheet.xsl")

        template_set.logo = FileObject(Command.path_of_default_template_file("generic", "logo.jpg"))
        template_set.fopConfigurationFile = FileObject(Command.path_of_default_template_file("generic", "fontconfig.xml"))
        template_set.bankingaccountref = "xx-xxxxxx-x"
        template_set.addresser = _("John Smit, Sample Company, 8976 Smallville")
        template_set.headerTextsalesorders = _(
            "According to your wishes the contract consists of the following positions:")
        template_set.footerTextsalesorders = _("Thank you for your interest in our company \n Best regards")
        template_set.headerTextpurchaseorders = _("We would like to order the following positions:")
        template_set.footerTextpurchaseorders = _("Best regards")
        template_set.pagefooterleft = _("Sample Company")
        template_set.pagefootermiddle = _("Sample Address")
        template_set.save()
        currency = crm.models.Currency()
        currency.description = "US Dollar"
        currency.shortName = "USD"
        currency.rounding = "0.10"
        currency.save()
        user_extension = djangoUserExtension.models.UserExtension()
        user_extension.defaultTemplateSet = template_set
        user_extension.defaultCurrency = currency
        user_extension.user = User.objects.all()[0]
        user_extension.save()
        postaladdress = djangoUserExtension.models.UserExtensionPostalAddress()
        postaladdress.purpose = 'H'
        postaladdress.name = "John"
        postaladdress.prename = "Smith"
        postaladdress.addressline1 = "Ave 1"
        postaladdress.zipcode = 899887
        postaladdress.town = "Smallville"
        postaladdress.userExtension = user_extension
        postaladdress.save()
        phoneaddress = djangoUserExtension.models.UserExtensionPhoneAddress()
        phoneaddress.phone = "1293847"
        phoneaddress.purpose = 'H'
        phoneaddress.userExtension = user_extension
        phoneaddress.save()
        emailaddress = djangoUserExtension.models.UserExtensionEmailAddress()
        emailaddress.email = "john.smith@smallville.com"
        emailaddress.purpose = 'H'
        emailaddress.userExtension = user_extension
        emailaddress.save()



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext as _


class Product(models.Model):
    identifier = models.CharField(verbose_name=_("Product Number"),
                                  max_length=200,
                                  null=True,
                                  blank=True)
    product_type = models.ForeignKey("ProductType", on_delete=models.CASCADE, verbose_name=_("Product Type"))




# -*- coding: utf-8 -*-

from koalixcrm.crm.models import PaymentReminder
from koalixcrm.crm.factories.factory_sales_document import StandardSalesDocumentFactory


class StandardPaymentReminderFactory(StandardSalesDocumentFactory):
    class Meta:
        model = PaymentReminder

    payable_until = "2018-05-20"
    payment_bank_reference = "This is a bank account reference"
    iteration_number = "1"
    status = "C"



# -*- coding: utf-8 -*-

import factory
from django.contrib.auth.models import User


class StaffUserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User
        django_get_or_create = ('username',)

    username = 'staff_user'
    email = 'staff_user@staff_user.com'

    password = factory.PostGenerationMethodCall('set_password', 'staff_user')

    is_superuser = False
    is_staff = True
    is_active = True


class AdminUserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User
        django_get_or_create = ('username',)

    username = 'admin'
    email = 'admin@admin.com'

    password = factory.PostGenerationMethodCall('set_password', 'admin')

    is_superuser = True
    is_staff = True
    is_active = True


class StandardUserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User
        django_get_or_create = ('username',)

    username = 'standard'
    email = 'standard@standard.com'

    password = factory.PostGenerationMethodCall('set_password', 'standard')

    is_superuser = False
    is_staff = False
    is_active = True



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-12 20:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounting', '0007_auto_20180422_2105'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, verbose_name='Product Category Title')),
            ],
            options={
                'verbose_name': 'Product Category',
                'verbose_name_plural': 'Product Categories',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-12 19:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0030_auto_20180611_1930'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='short_description',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Short Description'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-09 20:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0003_auto_20171009_1949'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contact',
            name='dateofcreation',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='lastmodification',
            field=models.DateTimeField(auto_now=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='dateofcreation',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='lastmodification',
            field=models.DateTimeField(auto_now=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='product',
            name='dateofcreation',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='product',
            name='lastmodification',
            field=models.DateTimeField(auto_now=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='dateofcreation',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='last_print_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Zuletzt gedruckt'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='lastmodification',
            field=models.DateTimeField(auto_now=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='dateofcreation',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='last_print_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Zuletzt gedruckt'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='lastmodification',
            field=models.DateTimeField(auto_now=True, verbose_name='Zuletzt geändert'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-12 19:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0032_auto_20180612_1925'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='projectstatus',
            options={'verbose_name': 'Project Status', 'verbose_name_plural': 'Project Status'},
        ),
        migrations.AlterField(
            model_name='task',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Project', verbose_name='Project'),
        ),
    ]



from rest_framework import serializers
from koalixcrm.crm.reporting.task_status import TaskStatus


class OptionTaskStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    isDone = serializers.BooleanField(source='is_done', read_only=True)

    class Meta:
        model = TaskStatus
        fields = ('title',
                  'description',
                  'isDone')


class TaskStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    isDone = serializers.BooleanField(source='is_done')

    class Meta:
        model = TaskStatus
        fields = ('title',
                  'description',
                  'isDone')



# -*- coding: utf-8 -*-

import factory
import datetime
from koalixcrm.crm.models import ProductPrice
from koalixcrm.crm.factories.factory_product import StandardProductFactory
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.global_support_functions import make_date_utc


class StandardPriceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProductPrice
        django_get_or_create = ('product_type',
                                'unit',
                                'currency',
                                'customer_group',
                                'price',
                                'valid_from',
                                'valid_until')

    product_type = factory.SubFactory(StandardProductFactory)
    unit = factory.SubFactory(StandardUnitFactory)
    currency = factory.SubFactory(StandardCurrencyFactory)
    customer_group = factory.SubFactory(StandardCustomerGroupFactory)
    price = "100.50"
    valid_from = make_date_utc(datetime.datetime(2018, 6, 15, 00)).date()
    valid_until = make_date_utc(datetime.datetime(2024, 6, 15, 00)).date()


class HighPriceFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProductPrice
    product_type = factory.SubFactory(StandardProductFactory)
    unit = factory.SubFactory(StandardUnitFactory)
    currency = factory.SubFactory(StandardCurrencyFactory)
    customer_group = factory.SubFactory(StandardCustomerGroupFactory)
    price = "250.50"
    valid_from = make_date_utc(datetime.datetime(2018, 6, 15, 00)).date()
    valid_until = make_date_utc(datetime.datetime(2024, 6, 15, 00)).date()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-26 20:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0035_auto_20180619_1924'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReportingPeriod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200, verbose_name='Title')),
                ('begin', models.DateField(verbose_name='Begin')),
                ('end', models.DateField(verbose_name='End')),
                ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Project', verbose_name='Project')),
            ],
            options={
                'verbose_name': 'Project',
                'verbose_name_plural': 'Projects',
            },
        ),
        migrations.AddField(
            model_name='work',
            name='reporting_period',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='crm.ReportingPeriod', verbose_name='Reporting Period'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-01 21:28
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0036_auto_20180626_2059'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='reportingperiod',
            options={'verbose_name': 'Reporting Period', 'verbose_name_plural': 'Reporting Periods'},
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _
from django.contrib import admin
from rest_framework import serializers


class AgreementStatus(models.Model):
    title = models.CharField(verbose_name=_("Title"),
                             max_length=250,
                             blank=False,
                             null=False)
    description = models.TextField(verbose_name=_("Text"),
                                   blank=True,
                                   null=True)
    is_agreed = models.BooleanField(verbose_name=_("Status represents agreement exists"),)

    class Meta:
        app_label = "crm"
        verbose_name = _('Agreement Status')
        verbose_name_plural = _('Agreement Status')

    def __str__(self):
        return str(self.id) + " " + str(self.title)


class AgreementStatusAdminView(admin.ModelAdmin):
    list_display = ('id',
                    'title',
                    'description',
                    'is_agreed')

    fieldsets = (
        (_('Agreement Status'), {
            'fields': ('title',
                       'description',
                       'is_agreed')
        }),
    )
    save_as = True


class AgreementStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = AgreementStatus
        fields = ('id',
                  'title',
                  'description',)
